/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickDC;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the DC Brick.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickDCActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set Velocity")
    public void brickDCSetVelocity(@ActionInput(name = "velocity") short velocity) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setVelocity(velocity);
    }

    public static void brickDCSetVelocity(@Nullable ThingActions actions, short velocity) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetVelocity(velocity);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Velocity")
    public @ActionOutput(name = "velocity", type = "short") Map<String, Object> brickDCGetVelocity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getVelocity();

        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickDCGetVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Current Velocity")
    public @ActionOutput(name = "velocity", type = "short") Map<String, Object> brickDCGetCurrentVelocity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getCurrentVelocity();

        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickDCGetCurrentVelocity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetCurrentVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Set Acceleration")
    public void brickDCSetAcceleration(@ActionInput(name = "acceleration") int acceleration)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setAcceleration(acceleration);
    }

    public static void brickDCSetAcceleration(@Nullable ThingActions actions, int acceleration)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetAcceleration(acceleration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "acceleration", type = "int") Map<String, Object> brickDCGetAcceleration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getAcceleration();

        result.put("acceleration", value);
        return result;
    }

    public static Map<String, Object> brickDCGetAcceleration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetAcceleration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Full Brake")
    public void brickDCFullBrake() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.fullBrake();
    }

    public static void brickDCFullBrake(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCFullBrake();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Enable")
    public void brickDCEnable() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.enable();
    }

    public static void brickDCEnable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCEnable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Disable")
    public void brickDCDisable() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.disable();
    }

    public static void brickDCDisable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCDisable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type = "boolean") Map<String, Object> brickDCIsEnabled()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isEnabled();

        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickDCIsEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCIsEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Set PWM Frequency")
    public void brickDCSetPWMFrequency(@ActionInput(name = "frequency") int frequency) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setPWMFrequency(frequency);
    }

    public static void brickDCSetPWMFrequency(@Nullable ThingActions actions, int frequency)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetPWMFrequency(frequency);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get PWM Frequency")
    public @ActionOutput(name = "frequency", type = "int") Map<String, Object> brickDCGetPWMFrequency()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getPWMFrequency();

        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> brickDCGetPWMFrequency(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetPWMFrequency();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Stack Input Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickDCGetStackInputVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStackInputVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetStackInputVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetStackInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get External Input Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickDCGetExternalInputVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getExternalInputVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetExternalInputVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetExternalInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Current Consumption")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickDCGetCurrentConsumption()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getCurrentConsumption();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetCurrentConsumption(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetCurrentConsumption();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Set Drive Mode")
    public void brickDCSetDriveMode(@ActionInput(name = "mode") short mode) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setDriveMode(mode);
    }

    public static void brickDCSetDriveMode(@Nullable ThingActions actions, short mode) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetDriveMode(mode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Drive Mode")
    public @ActionOutput(name = "mode", type = "short") Map<String, Object> brickDCGetDriveMode()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getDriveMode();

        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickDCGetDriveMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetDriveMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Minimum Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickDCGetMinimumVoltage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getMinimumVoltage();

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetMinimumVoltage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetMinimumVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickDCGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickDC.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickDCGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Is Status LED Enabled")
    public @ActionOutput(name = "enabled", type = "boolean") Map<String, Object> brickDCIsStatusLEDEnabled()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isStatusLEDEnabled();

        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickDCIsStatusLEDEnabled(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCIsStatusLEDEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "short") Map<String, Object> brickDCGetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickDCGetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Send Timeout Count")
    public @ActionOutput(name = "timeoutCount", type = "long") Map<String, Object> brickDCGetSendTimeoutCount(
            @ActionInput(name = "communicationMethod") short communicationMethod) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.getSendTimeoutCount(communicationMethod);

        result.put("timeoutCount", value);
        return result;
    }

    public static Map<String, Object> brickDCGetSendTimeoutCount(@Nullable ThingActions actions,
            short communicationMethod) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSendTimeoutCount(communicationMethod);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate Config")
    public @ActionOutput(name = "enableDynamicBaudrate", type = "boolean") @ActionOutput(name = "minimumDynamicBaudrate", type = "long") Map<String, Object> brickDCGetSPITFPBaudrateConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickDC.SPITFPBaudrateConfig value = dev.getSPITFPBaudrateConfig();

        result.put("enableDynamicBaudrate", value.enableDynamicBaudrate);
        result.put("minimumDynamicBaudrate", value.minimumDynamicBaudrate);
        return result;
    }

    public static Map<String, Object> brickDCGetSPITFPBaudrateConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSPITFPBaudrateConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountACKChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickDCGetSPITFPErrorCount(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickDC.SPITFPErrorCount value = dev.getSPITFPErrorCount(brickletPort);

        result.put("errorCountACKChecksum", value.errorCountACKChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickDCGetSPITFPErrorCount(@Nullable ThingActions actions, char brickletPort)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSPITFPErrorCount(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate")
    public @ActionOutput(name = "baudrate", type = "long") Map<String, Object> brickDCGetSPITFPBaudrate(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickDCWrapper dev = ((@Nullable BrickDCWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.getSPITFPBaudrate(brickletPort);

        result.put("baudrate", value);
        return result;
    }

    public static Map<String, Object> brickDCGetSPITFPBaudrate(@Nullable ThingActions actions, char brickletPort)
            throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSPITFPBaudrate(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }
}
