/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickDC;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the DC Brick.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickDCWrapper extends BrickDC implements DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickDC.DEVICE_DISPLAY_NAME, "brickdc",
            BrickDC.DEVICE_IDENTIFIER, BrickDCWrapper.class, BrickDCActions.class, "2.3.5", false);

    public BrickDCWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickDCWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickDCConfig cfg = (BrickDCConfig) config.as(BrickDCConfig.class);
        this.addVelocityReachedListener(this.reg((velocity) -> {
            if (true) {
                triggerChannelFn.accept("BrickDCVelocityReached",
                        transformBrickDCVelocityReachedCallback0(velocity, cfg));
            }
        }, this::removeVelocityReachedListener));
        this.addEmergencyShutdownListener(this.reg(() -> {
            if (true) {
                triggerChannelFn.accept("BrickDCEmergencyShutdown", transformBrickDCEmergencyShutdownCallback0(cfg));
            }
        }, this::removeEmergencyShutdownListener));
        this.addUnderVoltageListener(this.reg((voltage) -> {
            if (true) {
                triggerChannelFn.accept("BrickDCUnderVoltage", transformBrickDCUnderVoltageCallback0(voltage, cfg));
            }
        }, this::removeUnderVoltageListener));
        this.addCurrentVelocityListener(this.reg((velocity) -> {
            if (true) {
                updateStateFn.accept("BrickDCCurrentVelocity", transformBrickDCCurrentVelocityCallback0(velocity, cfg));
            }
        }, this::removeCurrentVelocityListener));
        this.setMinimumVoltage((int) (cfg.minimumVoltage.doubleValue() * 1000.0));
        if (cfg.statusLEDConfig) {
            this.enableStatusLED();
        } else {
            this.disableStatusLED();
        }
        this.setSPITFPBaudrateConfig(cfg.spitfpEnableDynamicBaudrate, cfg.spitfpMinimumDynamicBaudrate);
        this.setSPITFPBaudrate('a', cfg.spitfpBaudratePortA);
        this.setSPITFPBaudrate('b', cfg.spitfpBaudratePortB);
        {
            BrickDCCurrentVelocityConfig channelCfg = getChannelConfigFn.apply("BrickDCCurrentVelocity")
                    .as(BrickDCCurrentVelocityConfig.class);
            this.setCurrentVelocityPeriod(channelCfg.updateInterval);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickDCVelocityReached");
        result.add("BrickDCEmergencyShutdown");
        result.add("BrickDCUnderVoltage");
        result.add("BrickDCCurrentVelocity");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickDCCurrentVelocity":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickDCCurrentVelocity"), "Current Velocity",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickDCCurrentVelocity"))
                        .withDescription("The current velocity of the motor.")
                        .withStateDescription(StateDescriptionFragmentBuilder.create()
                                .withMinimum(BigDecimal.valueOf(-100)).withMaximum(BigDecimal.valueOf(100))
                                .withStep(BigDecimal.valueOf(0.0030518509475997192)).withPattern("%.3f %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge DC Brick").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Drives one brushed DC motor with up to 28V and 5A (peak)")
                .withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickDCVelocityReached",
                        new ChannelTypeUID("system", "trigger")).withLabel("Velocity Reached").withDescription(
                                "This channel is triggered whenever a set velocity is reached. For example: If a velocity of 0 is present, acceleration is set to 5000 and velocity to 10000, the channel will be triggered after about 2 seconds, when the set velocity is actually reached.<br/><br/>Note<br/><br/>Since we can't get any feedback from the DC motor, this only works if the acceleration is set smaller or equal to the maximum acceleration of the motor. Otherwise the motor will lag behind the control value and the listener will be triggered too early.")
                                .build(),
                        new ChannelDefinitionBuilder("BrickDCEmergencyShutdown", new ChannelTypeUID("system",
                                "trigger")).withLabel("Emergency Shutdown").withDescription(
                                        "This channel is triggered if either the current consumption is too high (above 5A) or the temperature of the driver chip is too high (above 175°C). These two possibilities are essentially the same, since the temperature will reach this threshold immediately if the motor consumes too much current. In case of a voltage below 3.3V (external or stack) this listener is triggered as well.<br/><br/>If this listener is triggered, the driver chip gets disabled at the same time. That means, the enable-action has to be called to drive the motor again.<br/><br/>Note<br/><br/>This listener only works in Drive/Brake mode. In Drive/Coast mode it is unfortunately impossible to reliably read the overcurrent/overtemperature signal from the driver chip.")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickDCUnderVoltage", new ChannelTypeUID("system", "trigger"))
                                .withLabel("Under Voltage")
                                .withDescription(
                                        "This channel is triggered when the input voltage drops below the configured minimum voltage. The parameter is the current voltage.")
                                .build(),
                        new ChannelDefinitionBuilder("BrickDCCurrentVelocity",
                                new ChannelTypeUID("tinkerforge", "BrickDCCurrentVelocity"))
                                        .withLabel("Current Velocity").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickdc":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("minimumVoltage", Type.DECIMAL).withDefault("6")
                                .withDescription(
                                        "The minimum voltage in V, below which the Under Voltage channel is triggered. The minimum possible value that works with the DC Brick is 5V. You can use this function to detect the discharge of a battery that is used to drive the stepper motor. If you have a fixed power supply, you likely do not need this functionality. The default value is 5V.")
                                .withLabel("Minimum Voltage").withUnit("V").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(65.535)).build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.BOOLEAN).withDefault("true")
                                .withDescription(
                                        "The status LED is the blue LED next to the USB connector. If enabled is is on and it flickers if data is transfered. If disabled it is always off.")
                                .withLabel("Status LED Config").build(),
                        ConfigDescriptionParameterBuilder.create("spitfpEnableDynamicBaudrate", Type.BOOLEAN)
                                .withDefault("true")
                                .withDescription(
                                        "The SPITF protocol can be used with a dynamic baudrate. If the dynamic baudrate is enabled, the Brick will try to adapt the baudrate for the communication between Bricks and Bricklets according to the amount of data that is transferred.<br/><br/>The baudrate will be increased exponentially if lots of data is sent/received and decreased linearly if little data is sent/received.<br/><br/>This lowers the baudrate in applications where little data is transferred (e.g. a weather station) and increases the robustness. If there is lots of data to transfer (e.g. Thermal Imaging Bricklet) it automatically increases the baudrate as needed.<br/><br/>In cases where some data has to transferred as fast as possible every few seconds (e.g. RS485 Bricklet with a high baudrate but small payload) you may want to turn the dynamic baudrate off to get the highest possible performance.<br/><br/>The maximum value of the baudrate can be set per port. If the dynamic baudrate is disabled, the maximum baudrate will be used statically.")
                                .withLabel("SPITFP Enable Dynamic Baudrate").withAdvanced(true).build(),
                        ConfigDescriptionParameterBuilder.create("spitfpMinimumDynamicBaudrate", Type.INTEGER)
                                .withDefault("400000").withDescription("See SPITFP Enable Dynamic Baudrate")
                                .withLabel("SPITFP Minimum Dynamic Baudrate").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("spitfpBaudratePortA", Type.INTEGER)
                                .withDefault("1400000")
                                .withDescription(
                                        "The baudrate for Bricklet port A.<br/><br/>If you want to increase the throughput of Bricklets you can increase the baudrate. If you get a high error count because of high interference you can decrease the baudrate.<br/><br/>If the dynamic baudrate feature is enabled, this is the maximum baudrate.<br/><br/>Regulatory testing is done with the default baudrate. If CE compatibility or similar is necessary in you applications we recommend to not change the baudrate.")
                                .withLabel("(Maximum) SPITFP Baudrate Port A").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("spitfpBaudratePortB", Type.INTEGER)
                                .withDefault("1400000")
                                .withDescription(
                                        "The baudrate for Bricklet port B.<br/><br/>If you want to increase the throughput of Bricklets you can increase the baudrate. If you get a high error count because of high interference you can decrease the baudrate.<br/><br/>If the dynamic baudrate feature is enabled, this is the maximum baudrate.<br/><br/>Regulatory testing is done with the default baudrate. If CE compatibility or similar is necessary in you applications we recommend to not change the baudrate.")
                                .withLabel("(Maximum) SPITFP Baudrate Port B").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickDCCurrentVelocity":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(65535)).build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickDCConfig cfg = (BrickDCConfig) config.as(BrickDCConfig.class);
        switch (channel) {
            case "BrickDCVelocityReached":
                return;
            case "BrickDCEmergencyShutdown":
                return;
            case "BrickDCUnderVoltage":
                return;
            case "BrickDCCurrentVelocity": {
                short value = this.getCurrentVelocity();
                if (true)
                    updateStateFn.accept(channel, transformBrickDCCurrentVelocityGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private String transformBrickDCVelocityReachedCallback0(short velocity, BrickDCConfig cfg) {
        return "";
    }

    private String transformBrickDCEmergencyShutdownCallback0(BrickDCConfig cfg) {
        return "";
    }

    private String transformBrickDCUnderVoltageCallback0(int voltage, BrickDCConfig cfg) {
        return "";
    }

    private org.eclipse.smarthome.core.types.State transformBrickDCCurrentVelocityCallback0(short velocity,
            BrickDCConfig cfg) {
        return new DecimalType(velocity);
    }

    private org.eclipse.smarthome.core.types.State transformBrickDCCurrentVelocityGetter0(short value,
            BrickDCConfig cfg) {
        return new DecimalType(value);
    }
}
