/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletCO2V2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the CO2 Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletCO2V2Wrapper extends BrickletCO2V2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletCO2V2.DEVICE_DISPLAY_NAME, "brickletco2v2",
            BrickletCO2V2.DEVICE_IDENTIFIER, BrickletCO2V2Wrapper.class, BrickletCO2V2Actions.class, "2.0.0", true);

    public BrickletCO2V2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletCO2V2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletCO2V2Config cfg = (BrickletCO2V2Config) config.as(BrickletCO2V2Config.class);
        this.addCO2ConcentrationListener(this.reg((co2Concentration) -> {
            if (true) {
                updateStateFn.accept("BrickletCO2V2CO2Concentration",
                        transformBrickletCO2V2CO2ConcentrationCallback0(co2Concentration, cfg));
            }
        }, this::removeCO2ConcentrationListener));
        this.addTemperatureListener(this.reg((temperature) -> {
            if (true) {
                updateStateFn.accept("BrickletCO2V2Temperature",
                        transformBrickletCO2V2TemperatureCallback0(temperature, cfg));
            }
        }, this::removeTemperatureListener));
        this.addHumidityListener(this.reg((humidity) -> {
            if (true) {
                updateStateFn.accept("BrickletCO2V2Humidity", transformBrickletCO2V2HumidityCallback0(humidity, cfg));
            }
        }, this::removeHumidityListener));

        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletCO2V2CO2ConcentrationConfig channelCfg = getChannelConfigFn.apply("BrickletCO2V2CO2Concentration")
                    .as(BrickletCO2V2CO2ConcentrationConfig.class);
            this.setCO2ConcentrationCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletCO2V2TemperatureConfig channelCfg = getChannelConfigFn.apply("BrickletCO2V2Temperature")
                    .as(BrickletCO2V2TemperatureConfig.class);
            this.setTemperatureCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletCO2V2HumidityConfig channelCfg = getChannelConfigFn.apply("BrickletCO2V2Humidity")
                    .as(BrickletCO2V2HumidityConfig.class);
            this.setHumidityCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setCO2ConcentrationCallbackConfiguration(0, true, 'x', 0, 0);
        this.setTemperatureCallbackConfiguration(0, true, 'x', 0, 0);
        this.setHumidityCallbackConfiguration(0, true, 'x', 0, 0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletCO2V2CO2Concentration");
        result.add("BrickletCO2V2Temperature");
        result.add("BrickletCO2V2Humidity");
        result.add("BrickletCO2V2AirPressure");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletCO2V2CO2Concentration":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCO2V2CO2Concentration"), "CO2 Concentration",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCO2V2CO2Concentration"))
                        .withDescription("The measured CO2 concentration.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(40000)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletCO2V2Temperature":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCO2V2Temperature"), "Temperature",
                                "Number:Temperature")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCO2V2Temperature"))
                        .withDescription("The measured temperature.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-40))
                                        .withMaximum(BigDecimal.valueOf(120)).withStep(BigDecimal.valueOf(0.01))
                                        .withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletCO2V2Humidity":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCO2V2Humidity"), "Humidity",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCO2V2Humidity"))
                        .withDescription("The measured humidity.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(100)).withStep(BigDecimal.valueOf(0.01))
                                        .withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletCO2V2AirPressure":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCO2V2AirPressure"), "Air Pressure",
                                "Number:Pressure")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCO2V2AirPressure"))
                        .withDescription(
                                "The CO2 concentration (among other things) depends on the ambient air pressure. To increase the accuracy of the CO2 Bricklet 2.0 you can set the current air pressure. You use the Barometer Bricklet 2.0 or the Air Quality Bricklet to get the current air pressure. The expected unit of the ambient air pressure value is bar. By default air pressure compensation is disabled. Once you set a value it will be used for compensation. You can turn the compensation off again by setting the value to 0. It is sufficient to update the value every few minutes.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(1.2)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(false).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge CO2 Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(
                        URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures CO2 concentration, temperature and humidity")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletCO2V2CO2Concentration",
                                new ChannelTypeUID("tinkerforge", "BrickletCO2V2CO2Concentration"))
                                        .withLabel("CO2 Concentration").build(),
                        new ChannelDefinitionBuilder("BrickletCO2V2Temperature",
                                new ChannelTypeUID("tinkerforge", "BrickletCO2V2Temperature")).withLabel("Temperature")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletCO2V2Humidity",
                                new ChannelTypeUID("tinkerforge", "BrickletCO2V2Humidity")).withLabel("Humidity")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletCO2V2AirPressure",
                                new ChannelTypeUID("tinkerforge", "BrickletCO2V2AirPressure")).withLabel("Air Pressure")
                                        .build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletco2v2":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletCO2V2CO2Concentration":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletCO2V2Temperature":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletCO2V2Humidity":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletCO2V2AirPressure":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletCO2V2Config cfg = (BrickletCO2V2Config) config.as(BrickletCO2V2Config.class);
        switch (channel) {
            case "BrickletCO2V2CO2Concentration": {
                int value = this.getCO2Concentration();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCO2V2CO2ConcentrationGetter0(value, cfg));
            }
                return;
            case "BrickletCO2V2Temperature": {
                int value = this.getTemperature();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCO2V2TemperatureGetter0(value, cfg));
            }
                return;
            case "BrickletCO2V2Humidity": {
                int value = this.getHumidity();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCO2V2HumidityGetter0(value, cfg));
            }
                return;
            case "BrickletCO2V2AirPressure": {
                int value = this.getAirPressure();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCO2V2AirPressureGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickletCO2V2AirPressure":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
                    this.setAirPressure((int) Math.round(cmd.doubleValue() * 1000.0));
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCO2V2CO2ConcentrationCallback0(int co2Concentration,
            BrickletCO2V2Config cfg) {
        return new QuantityType<>(co2Concentration, SmartHomeUnits.PARTS_PER_MILLION);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCO2V2TemperatureCallback0(int temperature,
            BrickletCO2V2Config cfg) {
        return new QuantityType<>(temperature / 100.0, SIUnits.CELSIUS);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCO2V2HumidityCallback0(int humidity,
            BrickletCO2V2Config cfg) {
        return new QuantityType<>(humidity / 100.0, SmartHomeUnits.PERCENT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCO2V2CO2ConcentrationGetter0(int value,
            BrickletCO2V2Config cfg) {
        return new QuantityType<>(value, SmartHomeUnits.PARTS_PER_MILLION);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCO2V2TemperatureGetter0(int value,
            BrickletCO2V2Config cfg) {
        return new QuantityType<>(value / 100.0, SIUnits.CELSIUS);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCO2V2HumidityGetter0(int value,
            BrickletCO2V2Config cfg) {
        return new QuantityType<>(value / 100.0, SmartHomeUnits.PERCENT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCO2V2AirPressureGetter0(int value,
            BrickletCO2V2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.BAR);
    }
}
