/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletCompass;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Compass Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletCompassWrapper extends BrickletCompass implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletCompass.DEVICE_DISPLAY_NAME, "brickletcompass",
            BrickletCompass.DEVICE_IDENTIFIER, BrickletCompassWrapper.class, BrickletCompassActions.class, "2.0.0",
            true);

    public BrickletCompassWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletCompassWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletCompassConfig cfg = (BrickletCompassConfig) config.as(BrickletCompassConfig.class);
        this.addHeadingListener(this.reg((heading) -> {
            if (true) {
                updateStateFn.accept("BrickletCompassHeading", transformBrickletCompassHeadingCallback0(heading, cfg));
            }
        }, this::removeHeadingListener));
        this.addMagneticFluxDensityListener(this.reg((x, y, z) -> {
            if (true) {
                updateStateFn.accept("BrickletCompassMagneticFluxDensityX",
                        transformBrickletCompassMagneticFluxDensityXCallback0(x, y, z, cfg));
            }
        }, this::removeMagneticFluxDensityListener));
        this.addMagneticFluxDensityListener(this.reg((x, y, z) -> {
            if (true) {
                updateStateFn.accept("BrickletCompassMagneticFluxDensityY",
                        transformBrickletCompassMagneticFluxDensityYCallback0(x, y, z, cfg));
            }
        }, this::removeMagneticFluxDensityListener));
        this.addMagneticFluxDensityListener(this.reg((x, y, z) -> {
            if (true) {
                updateStateFn.accept("BrickletCompassMagneticFluxDensityZ",
                        transformBrickletCompassMagneticFluxDensityZCallback0(x, y, z, cfg));
            }
        }, this::removeMagneticFluxDensityListener));
        this.setConfiguration(cfg.dataRate, cfg.backgroundCalibration);
        this.setMagneticFluxDensityCallbackConfiguration(cfg.magneticFluxDensityUpdateInterval, true);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletCompassHeadingConfig channelCfg = getChannelConfigFn.apply("BrickletCompassHeading")
                    .as(BrickletCompassHeadingConfig.class);
            this.setHeadingCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setHeadingCallbackConfiguration(0, true, 'x', 0, 0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletCompassHeading");
        result.add("BrickletCompassMagneticFluxDensityX");
        result.add("BrickletCompassMagneticFluxDensityY");
        result.add("BrickletCompassMagneticFluxDensityZ");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletCompassHeading":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCompassHeading"), "Heading", "Number:Angle")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletCompassHeading"))
                        .withDescription("The heading (north = 0 degree)")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(360)).withStep(BigDecimal.valueOf(0.1))
                                        .withPattern("%.1f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletCompassMagneticFluxDensity":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletCompassMagneticFluxDensity"),
                                "Magnetic Flux Density", "Number:MagneticFluxDensity")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletCompassMagneticFluxDensity"))
                        .withDescription("The heading (north = 0 degree)")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-0.0008))
                                        .withMaximum(BigDecimal.valueOf(0.0008)).withStep(BigDecimal.valueOf(1e-08))
                                        .withPattern("%.8f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Compass Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("3-axis compass with 10 nanotesla and 0.1° resolution")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletCompassHeading",
                                new ChannelTypeUID("tinkerforge", "BrickletCompassHeading")).withLabel("Heading")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletCompassMagneticFluxDensityX",
                                new ChannelTypeUID("tinkerforge", "BrickletCompassMagneticFluxDensity"))
                                        .withLabel("Magnetic Flux Density - X")
                                        .withDescription(
                                                "The magnetic flux density (magnetic induction): https://en.wikipedia.org/wiki/Magnetic_flux measured in direction of the x axis.")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletCompassMagneticFluxDensityY",
                                new ChannelTypeUID("tinkerforge", "BrickletCompassMagneticFluxDensity"))
                                        .withLabel("Magnetic Flux Density - Y")
                                        .withDescription(
                                                "The magnetic flux density (magnetic induction): https://en.wikipedia.org/wiki/Magnetic_flux measured in direction of the y axis.")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletCompassMagneticFluxDensityZ",
                                new ChannelTypeUID("tinkerforge", "BrickletCompassMagneticFluxDensity"))
                                        .withLabel("Magnetic Flux Density - Z")
                                        .withDescription(
                                                "The magnetic flux density (magnetic induction): https://en.wikipedia.org/wiki/Magnetic_flux measured in direction of the z axis.")
                                        .build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletcompass":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("dataRate", Type.INTEGER).withDefault("0")
                                .withDescription(
                                        "The data rate that is used by the magnetometer. The lower the data rate, the lower is the noise on the data.")
                                .withLabel("Data Rate").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "100Hz"),
                                        new ParameterOption("1", "200Hz"), new ParameterOption("2", "400Hz"),
                                        new ParameterOption("3", "600Hz")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("backgroundCalibration", Type.BOOLEAN)
                                .withDefault("true")
                                .withDescription(
                                        "If the background calibration is enabled the sensing polarity is flipped once per second to automatically calculate and remove offset that is caused by temperature changes. This polarity flipping takes about 20ms. This means that once a second you will not get new data for a period of 20ms. We highly recommend that you keep the background calibration enabled and only disable it if the 20ms off-time is a problem in you application.")
                                .withLabel("Enable Background Calibration").build(),
                        ConfigDescriptionParameterBuilder.create("magneticFluxDensityUpdateInterval", Type.INTEGER)
                                .withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval for the magnetic flux densities in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Magnetic Flux Density Update Interval")
                                .withUnit("ms").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(4294967295L)).build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletCompassHeading":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletCompassMagneticFluxDensity":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletCompassConfig cfg = (BrickletCompassConfig) config.as(BrickletCompassConfig.class);
        switch (channel) {
            case "BrickletCompassHeading": {
                int value = this.getHeading();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCompassHeadingGetter0(value, cfg));
            }
                return;
            case "BrickletCompassMagneticFluxDensityX": {
                BrickletCompass.MagneticFluxDensity value = this.getMagneticFluxDensity();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCompassMagneticFluxDensityXGetter0(value, cfg));
            }
                return;
            case "BrickletCompassMagneticFluxDensityY": {
                BrickletCompass.MagneticFluxDensity value = this.getMagneticFluxDensity();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCompassMagneticFluxDensityYGetter0(value, cfg));
            }
                return;
            case "BrickletCompassMagneticFluxDensityZ": {
                BrickletCompass.MagneticFluxDensity value = this.getMagneticFluxDensity();
                if (true)
                    updateStateFn.accept(channel, transformBrickletCompassMagneticFluxDensityZGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassHeadingCallback0(int heading,
            BrickletCompassConfig cfg) {
        return new QuantityType<>(heading / 10.0, SmartHomeUnits.DEGREE_ANGLE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassMagneticFluxDensityXCallback0(int x, int y,
            int z, BrickletCompassConfig cfg) {
        return new QuantityType<>(x, SmartHomeUnits.TESLA);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassMagneticFluxDensityYCallback0(int x, int y,
            int z, BrickletCompassConfig cfg) {
        return new QuantityType<>(y, SmartHomeUnits.TESLA);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassMagneticFluxDensityZCallback0(int x, int y,
            int z, BrickletCompassConfig cfg) {
        return new QuantityType<>(z, SmartHomeUnits.TESLA);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassHeadingGetter0(int value,
            BrickletCompassConfig cfg) {
        return new QuantityType<>(value / 10.0, SmartHomeUnits.DEGREE_ANGLE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassMagneticFluxDensityXGetter0(
            BrickletCompass.MagneticFluxDensity value, BrickletCompassConfig cfg) {
        return new QuantityType<>(value.x, SmartHomeUnits.TESLA);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassMagneticFluxDensityYGetter0(
            BrickletCompass.MagneticFluxDensity value, BrickletCompassConfig cfg) {
        return new QuantityType<>(value.y, SmartHomeUnits.TESLA);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletCompassMagneticFluxDensityZGetter0(
            BrickletCompass.MagneticFluxDensity value, BrickletCompassConfig cfg) {
        return new QuantityType<>(value.z, SmartHomeUnits.TESLA);
    }
}
