/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletDualButtonV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Dual Button Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletDualButtonV2Wrapper extends BrickletDualButtonV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletDualButtonV2.DEVICE_DISPLAY_NAME,
            "brickletdualbuttonv2", BrickletDualButtonV2.DEVICE_IDENTIFIER, BrickletDualButtonV2Wrapper.class,
            BrickletDualButtonV2Actions.class, "2.0.0", true);

    public BrickletDualButtonV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletDualButtonV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletDualButtonV2Config cfg = (BrickletDualButtonV2Config) config.as(BrickletDualButtonV2Config.class);
        this.addStateChangedListener(this.reg((buttonL, buttonR, ledL, ledR) -> {
            if (true) {
                triggerChannelFn.accept("BrickletDualButtonV2LeftButton",
                        transformBrickletDualButtonV2LeftButtonCallback0(buttonL, buttonR, ledL, ledR, cfg));
            }
        }, this::removeStateChangedListener));
        this.addStateChangedListener(this.reg((buttonL, buttonR, ledL, ledR) -> {
            if (true) {
                triggerChannelFn.accept("BrickletDualButtonV2RightButton",
                        transformBrickletDualButtonV2RightButtonCallback0(buttonL, buttonR, ledL, ledR, cfg));
            }
        }, this::removeStateChangedListener));
        this.setLEDState(cfg.leftLEDState, cfg.rightLEDState);
        this.setStateChangedCallbackConfiguration(true);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setStateChangedCallbackConfiguration(false);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        BrickletDualButtonV2Config cfg = (BrickletDualButtonV2Config) config.as(BrickletDualButtonV2Config.class);
        List<String> result = new ArrayList<String>();
        result.add("BrickletDualButtonV2LeftButton");
        result.add("BrickletDualButtonV2RightButton");
        if (cfg.leftLEDState > 1) {
            result.add("BrickletDualButtonV2LeftLED");
        }
        if (cfg.rightLEDState > 1) {
            result.add("BrickletDualButtonV2RightLED");
        }
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletDualButtonV2LeftLED":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletDualButtonV2LeftLED"), "Left LED", "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletDualButtonV2LeftLED"))
                        .withDescription("Controls the left LED.").build();
            case "BrickletDualButtonV2RightLED":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletDualButtonV2RightLED"), "Right LED", "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletDualButtonV2RightLED"))
                        .withDescription("Controls the right LED.").build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Dual Button Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Two tactile buttons with built-in blue LEDs")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletDualButtonV2LeftButton",
                                new ChannelTypeUID("system", "rawbutton"))
                                        .withLabel("Left Button")
                                        .withDescription(
                                                "This channel triggers if the left button is pressed or released")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletDualButtonV2RightButton",
                                new ChannelTypeUID("system", "rawbutton"))
                                        .withLabel("Right Button")
                                        .withDescription(
                                                "This channel triggers if the right button is pressed or released")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletDualButtonV2LeftLED",
                                new ChannelTypeUID("tinkerforge", "BrickletDualButtonV2LeftLED")).withLabel("Left LED")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletDualButtonV2RightLED",
                                new ChannelTypeUID("tinkerforge", "BrickletDualButtonV2RightLED"))
                                        .withLabel("Right LED").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletdualbuttonv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("leftLEDState", Type.INTEGER).withDefault("1")
                                .withDescription(
                                        "<ul><li>Auto Toggle - Default On: Enables auto toggle. LED initially enabled</li><li>Auto Toggle - Default Off: Enables auto toggle. LED initially disabled</li><li>Channel - Default On: Creates a control channel. LED initially enabled.</li><li>Channel - Default Off: Creates a control channel. LED initially disabled.</li></ul>")
                                .withLabel("Left LED State")
                                .withOptions(Arrays.asList(new ParameterOption("0", "Auto Toggle - Default On"),
                                        new ParameterOption("1", "Auto Toggle - Default Off"),
                                        new ParameterOption("2", "Channel - Default On"),
                                        new ParameterOption("3", "Channel - Default Off")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("rightLEDState", Type.INTEGER).withDefault("1")
                                .withDescription(
                                        "<ul><li>Auto Toggle - Default On: Enables auto toggle. LED initially enabled</li><li>Auto Toggle - Default Off: Enables auto toggle. LED initially disabled</li><li>Channel - Default On: Creates a control channel. LED initially enabled.</li><li>Channel - Default Off: Creates a control channel. LED initially disabled.</li></ul>")
                                .withLabel("Right LED State")
                                .withOptions(Arrays.asList(new ParameterOption("0", "Auto Toggle - Default On"),
                                        new ParameterOption("1", "Auto Toggle - Default Off"),
                                        new ParameterOption("2", "Channel - Default On"),
                                        new ParameterOption("3", "Channel - Default Off")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletDualButtonV2LeftLED":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletDualButtonV2RightLED":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletDualButtonV2Config cfg = (BrickletDualButtonV2Config) config.as(BrickletDualButtonV2Config.class);
        switch (channel) {
            case "BrickletDualButtonV2LeftButton": {
                BrickletDualButtonV2.ButtonState value = this.getButtonState();
                if (true)
                    triggerChannelFn.accept(channel, transformBrickletDualButtonV2LeftButtonGetter0(value, cfg));
            }
                return;
            case "BrickletDualButtonV2RightButton": {
                BrickletDualButtonV2.ButtonState value = this.getButtonState();
                if (true)
                    triggerChannelFn.accept(channel, transformBrickletDualButtonV2RightButtonGetter0(value, cfg));
            }
                return;
            case "BrickletDualButtonV2LeftLED": {
                BrickletDualButtonV2.LEDState value = this.getLEDState();
                if (true)
                    updateStateFn.accept(channel, transformBrickletDualButtonV2LeftLEDGetter0(value, cfg));
            }
                return;
            case "BrickletDualButtonV2RightLED": {
                BrickletDualButtonV2.LEDState value = this.getLEDState();
                if (true)
                    updateStateFn.accept(channel, transformBrickletDualButtonV2RightLEDGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickletDualButtonV2LeftLED":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedLEDState(BrickletDualButtonV2.LED_LEFT,
                            cmd == OnOffType.ON ? BrickletDualButtonV2.LED_STATE_ON
                                    : BrickletDualButtonV2.LED_STATE_OFF);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletDualButtonV2RightLED":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedLEDState(BrickletDualButtonV2.LED_RIGHT,
                            cmd == OnOffType.ON ? BrickletDualButtonV2.LED_STATE_ON
                                    : BrickletDualButtonV2.LED_STATE_OFF);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private String transformBrickletDualButtonV2LeftButtonCallback0(int buttonL, int buttonR, int ledL, int ledR,
            BrickletDualButtonV2Config cfg) {
        return buttonL == BrickletDualButtonV2.BUTTON_STATE_PRESSED ? CommonTriggerEvents.PRESSED
                : CommonTriggerEvents.RELEASED;
    }

    private String transformBrickletDualButtonV2RightButtonCallback0(int buttonL, int buttonR, int ledL, int ledR,
            BrickletDualButtonV2Config cfg) {
        return buttonR == BrickletDualButtonV2.BUTTON_STATE_PRESSED ? CommonTriggerEvents.PRESSED
                : CommonTriggerEvents.RELEASED;
    }

    private String transformBrickletDualButtonV2LeftButtonGetter0(BrickletDualButtonV2.ButtonState value,
            BrickletDualButtonV2Config cfg) {
        return value.buttonL == BrickletDualButtonV2.BUTTON_STATE_PRESSED ? CommonTriggerEvents.PRESSED
                : CommonTriggerEvents.RELEASED;
    }

    private String transformBrickletDualButtonV2RightButtonGetter0(BrickletDualButtonV2.ButtonState value,
            BrickletDualButtonV2Config cfg) {
        return value.buttonR == BrickletDualButtonV2.BUTTON_STATE_PRESSED ? CommonTriggerEvents.PRESSED
                : CommonTriggerEvents.RELEASED;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletDualButtonV2LeftLEDGetter0(
            BrickletDualButtonV2.LEDState value, BrickletDualButtonV2Config cfg) {
        return value.ledL == BrickletDualButtonV2.LED_STATE_ON ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletDualButtonV2RightLEDGetter0(
            BrickletDualButtonV2.LEDState value, BrickletDualButtonV2Config cfg) {
        return value.ledR == BrickletDualButtonV2.LED_STATE_ON ? OnOffType.ON : OnOffType.OFF;
    }
}
