/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletDualRelay;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Dual Relay Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDualRelayActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set State")
    public void brickletDualRelaySetState(@ActionInput(name = "relay1") boolean relay1,
            @ActionInput(name = "relay2") boolean relay2) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletDualRelayWrapper dev = ((@Nullable BrickletDualRelayWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setState(relay1, relay2);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayRelay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayRelay2"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay2"), RefreshType.REFRESH);
        }
    }

    public static void brickletDualRelaySetState(@Nullable ThingActions actions, boolean relay1, boolean relay2)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            ((BrickletDualRelayActions) actions).brickletDualRelaySetState(relay1, relay2);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Set Selected State")
    public void brickletDualRelaySetSelectedState(@ActionInput(name = "relay") short relay,
            @ActionInput(name = "state") boolean state) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletDualRelayWrapper dev = ((@Nullable BrickletDualRelayWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setSelectedState(relay, state);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayRelay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayRelay2"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay2"), RefreshType.REFRESH);
        }
    }

    public static void brickletDualRelaySetSelectedState(@Nullable ThingActions actions, short relay, boolean state)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            ((BrickletDualRelayActions) actions).brickletDualRelaySetSelectedState(relay, state);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Set Monoflop")
    public void brickletDualRelaySetMonoflop(@ActionInput(name = "relay") short relay,
            @ActionInput(name = "state") boolean state, @ActionInput(name = "time") long time)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletDualRelayWrapper dev = ((@Nullable BrickletDualRelayWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setMonoflop(relay, state, time);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayRelay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayRelay2"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay1")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay1"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay2")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletDualRelayMonoflopRelay2"), RefreshType.REFRESH);
        }
    }

    public static void brickletDualRelaySetMonoflop(@Nullable ThingActions actions, short relay, boolean state,
            long time) throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            ((BrickletDualRelayActions) actions).brickletDualRelaySetMonoflop(relay, state, time);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Get State")
    public @ActionOutput(name = "relay1", type = "boolean") @ActionOutput(name = "relay2", type = "boolean") Map<String, Object> brickletDualRelayGetState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDualRelayWrapper dev = ((@Nullable BrickletDualRelayWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDualRelay.State value = dev.getState();

        result.put("relay1", value.relay1);
        result.put("relay2", value.relay2);
        return result;
    }

    public static Map<String, Object> brickletDualRelayGetState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            return ((BrickletDualRelayActions) actions).brickletDualRelayGetState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "state", type = "boolean") @ActionOutput(name = "time", type = "long") @ActionOutput(name = "timeRemaining", type = "long") Map<String, Object> brickletDualRelayGetMonoflop(
            @ActionInput(name = "relay") short relay) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDualRelayWrapper dev = ((@Nullable BrickletDualRelayWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDualRelay.Monoflop value = dev.getMonoflop(relay);

        result.put("state", value.state);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletDualRelayGetMonoflop(@Nullable ThingActions actions, short relay)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            return ((BrickletDualRelayActions) actions).brickletDualRelayGetMonoflop(relay);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletDualRelayGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletDualRelayWrapper dev = ((@Nullable BrickletDualRelayWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletDualRelay.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDualRelayGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletDualRelayActions) {
            return ((BrickletDualRelayActions) actions).brickletDualRelayGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualRelayActions class.");
        }
    }
}
