/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletHallEffect;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Hall Effect Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletHallEffectWrapper extends BrickletHallEffect implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletHallEffect.DEVICE_DISPLAY_NAME,
            "bricklethalleffect", BrickletHallEffect.DEVICE_IDENTIFIER, BrickletHallEffectWrapper.class,
            BrickletHallEffectActions.class, "2.0.0", false);

    public BrickletHallEffectWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletHallEffectWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletHallEffectConfig cfg = (BrickletHallEffectConfig) config.as(BrickletHallEffectConfig.class);
        this.addEdgeCountListener(this.reg((count, value) -> {
            if (true) {
                updateStateFn.accept("BrickletHallEffectEdgeCount",
                        transformBrickletHallEffectEdgeCountCallback0(count, value, cfg));
            }
        }, this::removeEdgeCountListener));

        {
            BrickletHallEffectEdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletHallEffectEdgeCount")
                    .as(BrickletHallEffectEdgeCountConfig.class);
            this.setEdgeCountConfig(cfg.edgeType.shortValue(), cfg.debounce.shortValue());
            this.setEdgeInterrupt(channelCfg.refreshCount);
        }
        {
            BrickletHallEffectMagneticFieldDetectedConfig channelCfg = getChannelConfigFn
                    .apply("BrickletHallEffectMagneticFieldDetected")
                    .as(BrickletHallEffectMagneticFieldDetectedConfig.class);

            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletHallEffectMagneticFieldDetected"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletHallEffectEdgeCount");
        result.add("BrickletHallEffectMagneticFieldDetected");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletHallEffectEdgeCount":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletHallEffectEdgeCount"), "Edge Count",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletHallEffectEdgeCount"))
                        .withDescription("The current value of the edge counter.")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletHallEffectMagneticFieldDetected":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletHallEffectMagneticFieldDetected"),
                                "Magnetic Field Detected", "Switch")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletHallEffectMagneticFieldDetected"))
                        .withDescription("Enabled if a magnetic field of 3.5 millitesla or greater is detected.")
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Hall Effect Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(
                        URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Detects presence of magnetic field")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletHallEffectEdgeCount",
                                new ChannelTypeUID("tinkerforge", "BrickletHallEffectEdgeCount"))
                                        .withLabel("Edge Count").build(),
                        new ChannelDefinitionBuilder("BrickletHallEffectMagneticFieldDetected",
                                new ChannelTypeUID("tinkerforge", "BrickletHallEffectMagneticFieldDetected"))
                                        .withLabel("Magnetic Field Detected").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:bricklethalleffect":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("edgeType", Type.INTEGER).withDefault("0")
                                .withDescription(
                                        "The edge type parameter configures if rising edges, falling edges or both are counted.")
                                .withLabel("Edge Type").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Rising"),
                                        new ParameterOption("1", "Falling"), new ParameterOption("2", "Both")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("debounce", Type.INTEGER).withDefault("100")
                                .withDescription("The debounce time in ms.").withLabel("Debounce Time")
                                .withAdvanced(true).withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(255)).build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletHallEffectEdgeCount":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("refreshCount", Type.INTEGER).withDefault("1")
                                .withDescription("The edge count value will be refreshed every n-th edge.")
                                .withLabel("Refresh Count").withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(4294967295L)).build(),
                        ConfigDescriptionParameterBuilder.create("resetOnRead", Type.BOOLEAN).withDefault("false")
                                .withDescription(
                                        "Enabling this will reset the edge counter after OpenHAB reads its value. Use this if you want relative edge counts per update.")
                                .withLabel("Reset Edge Count On Update").build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletHallEffectMagneticFieldDetected":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withLabel("Update Interval").withUnit("ms").build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletHallEffectConfig cfg = (BrickletHallEffectConfig) config.as(BrickletHallEffectConfig.class);
        switch (channel) {
            case "BrickletHallEffectEdgeCount": {
                BrickletHallEffectEdgeCountConfig channelCfg = channelConfig
                        .as(BrickletHallEffectEdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletHallEffectEdgeCountGetter0(value, cfg));
                }
                return;
            }
            case "BrickletHallEffectMagneticFieldDetected": {
                boolean value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletHallEffectMagneticFieldDetectedGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHallEffectEdgeCountCallback0(long count,
            boolean value, BrickletHallEffectConfig cfg) {
        return new DecimalType(count);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHallEffectEdgeCountGetter0(long value,
            BrickletHallEffectConfig cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHallEffectMagneticFieldDetectedGetter0(
            boolean value, BrickletHallEffectConfig cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }
}
