/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletHumidityV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Humidity Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletHumidityV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Humidity")
    public @ActionOutput(name = "humidity", type = "int") Map<String, Object> brickletHumidityV2GetHumidity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getHumidity();

        result.put("humidity", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetHumidity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetHumidity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletHumidityV2GetTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Set Heater Configuration")
    public void brickletHumidityV2SetHeaterConfiguration(@ActionInput(name = "heaterConfig") int heaterConfig)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setHeaterConfiguration(heaterConfig);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletHumidityV2Heater")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletHumidityV2Heater"), RefreshType.REFRESH);
        }
    }

    public static void brickletHumidityV2SetHeaterConfiguration(@Nullable ThingActions actions, int heaterConfig)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            ((BrickletHumidityV2Actions) actions).brickletHumidityV2SetHeaterConfiguration(heaterConfig);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get Heater Configuration")
    public @ActionOutput(name = "heaterConfig", type = "int") Map<String, Object> brickletHumidityV2GetHeaterConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getHeaterConfiguration();

        result.put("heaterConfig", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetHeaterConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetHeaterConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLengthHumidity", type = "int") @ActionOutput(name = "movingAverageLengthTemperature", type = "int") Map<String, Object> brickletHumidityV2GetMovingAverageConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletHumidityV2.MovingAverageConfiguration value = dev.getMovingAverageConfiguration();

        result.put("movingAverageLengthHumidity", value.movingAverageLengthHumidity);
        result.put("movingAverageLengthTemperature", value.movingAverageLengthTemperature);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetMovingAverageConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get Samples Per Second")
    public @ActionOutput(name = "sps", type = "int") Map<String, Object> brickletHumidityV2GetSamplesPerSecond()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getSamplesPerSecond();

        result.put("sps", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetSamplesPerSecond(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetSamplesPerSecond();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletHumidityV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletHumidityV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletHumidityV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletHumidityV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletHumidityV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletHumidityV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletHumidityV2Wrapper dev = ((@Nullable BrickletHumidityV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletHumidityV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletHumidityV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletHumidityV2Actions) {
            return ((BrickletHumidityV2Actions) actions).brickletHumidityV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityV2Actions class.");
        }
    }
}
