/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletHumidity;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Humidity Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletHumidityWrapper extends BrickletHumidity implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletHumidity.DEVICE_DISPLAY_NAME,
            "bricklethumidity", BrickletHumidity.DEVICE_IDENTIFIER, BrickletHumidityWrapper.class,
            BrickletHumidityActions.class, "2.0.0", false);

    public BrickletHumidityWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletHumidityWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletHumidityConfig cfg = (BrickletHumidityConfig) config.as(BrickletHumidityConfig.class);
        this.addHumidityListener(this.reg((humidity) -> {
            if (true) {
                updateStateFn.accept("BrickletHumidityHumidity",
                        transformBrickletHumidityHumidityCallback0(humidity, cfg));
            }
        }, this::removeHumidityListener));
        this.addAnalogValueListener(this.reg((value) -> {
            if (true) {
                updateStateFn.accept("BrickletHumidityAnalogValue",
                        transformBrickletHumidityAnalogValueCallback0(value, cfg));
            }
        }, this::removeAnalogValueListener));

        {
            BrickletHumidityHumidityConfig channelCfg = getChannelConfigFn.apply("BrickletHumidityHumidity")
                    .as(BrickletHumidityHumidityConfig.class);
            this.setHumidityCallbackPeriod(channelCfg.updateInterval);
            this.setHumidityCallbackThreshold('x', 0, 0);
        }
        {
            BrickletHumidityAnalogValueConfig channelCfg = getChannelConfigFn.apply("BrickletHumidityAnalogValue")
                    .as(BrickletHumidityAnalogValueConfig.class);
            this.setAnalogValueCallbackPeriod(channelCfg.updateInterval);
            this.setAnalogValueCallbackThreshold('x', (short) 0, (short) 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setHumidityCallbackPeriod(0);
        this.setAnalogValueCallbackPeriod(0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletHumidityHumidity");
        result.add("BrickletHumidityAnalogValue");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletHumidityHumidity":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletHumidityHumidity"), "Humidity",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletHumidityHumidity"))
                        .withDescription("Measured relative humidity")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(100)).withStep(BigDecimal.valueOf(0.1))
                                        .withPattern("%.1f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletHumidityAnalogValue":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletHumidityAnalogValue"), "Analog Value",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletHumidityAnalogValue"))
                        .withDescription(
                                "<br/>Returns the value as read by a 12-bit analog-to-digital converter.<br/><br/>.. note::<br/> The value returned by :func:Get Humidity` is averaged over several samples<br/> to yield less noise, while :func:`Get Analog Value` gives back raw<br/> unfiltered analog values. The returned humidity value is calibrated for<br/> room temperatures, if you use the sensor in extreme cold or extreme<br/> warm environments, you might want to calculate the humidity from<br/> the analog value yourself. See the `HIH 5030 datasheet<br/>: https://github.com/Tinkerforge/humidity-bricklet/raw/master/datasheets/hih-5030.pdf.<br/>")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(4095)).withStep(BigDecimal.valueOf(1))
                                        .withPattern("%d").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Humidity Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures relative humidity")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletHumidityHumidity",
                                new ChannelTypeUID("tinkerforge", "BrickletHumidityHumidity")).withLabel("Humidity")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletHumidityAnalogValue",
                                new ChannelTypeUID("tinkerforge", "BrickletHumidityAnalogValue"))
                                        .withLabel("Analog Value").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:bricklethumidity":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletHumidityHumidity":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletHumidityAnalogValue":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletHumidityConfig cfg = (BrickletHumidityConfig) config.as(BrickletHumidityConfig.class);
        switch (channel) {
            case "BrickletHumidityHumidity": {
                int value = this.getHumidity();
                if (true)
                    updateStateFn.accept(channel, transformBrickletHumidityHumidityGetter0(value, cfg));
            }
                return;
            case "BrickletHumidityAnalogValue": {
                int value = this.getAnalogValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletHumidityAnalogValueGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityHumidityCallback0(int humidity,
            BrickletHumidityConfig cfg) {
        return new QuantityType<>(humidity / 10.0, SmartHomeUnits.PERCENT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityAnalogValueCallback0(int value,
            BrickletHumidityConfig cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityHumidityGetter0(int value,
            BrickletHumidityConfig cfg) {
        return new QuantityType<>(value / 10.0, SmartHomeUnits.PERCENT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletHumidityAnalogValueGetter0(int value,
            BrickletHumidityConfig cfg) {
        return new DecimalType(value);
    }
}
