/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletIO16V2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the IO-16 Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIO16V2Wrapper extends BrickletIO16V2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIO16V2.DEVICE_DISPLAY_NAME, "brickletio16v2",
            BrickletIO16V2.DEVICE_IDENTIFIER, BrickletIO16V2Wrapper.class, BrickletIO16V2Actions.class, "2.0.0", true);

    public BrickletIO16V2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletIO16V2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIO16V2Config cfg = (BrickletIO16V2Config) config.as(BrickletIO16V2Config.class);
        if (cfg.pinConfiguration0 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 0) {
                    updateStateFn.accept("BrickletIO16V2InputPin0",
                            transformBrickletIO16V2InputPin0Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration1 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 1) {
                    updateStateFn.accept("BrickletIO16V2InputPin1",
                            transformBrickletIO16V2InputPin1Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration2 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 2) {
                    updateStateFn.accept("BrickletIO16V2InputPin2",
                            transformBrickletIO16V2InputPin2Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration3 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 3) {
                    updateStateFn.accept("BrickletIO16V2InputPin3",
                            transformBrickletIO16V2InputPin3Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration4 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 4) {
                    updateStateFn.accept("BrickletIO16V2InputPin4",
                            transformBrickletIO16V2InputPin4Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration5 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 5) {
                    updateStateFn.accept("BrickletIO16V2InputPin5",
                            transformBrickletIO16V2InputPin5Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration6 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 6) {
                    updateStateFn.accept("BrickletIO16V2InputPin6",
                            transformBrickletIO16V2InputPin6Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration7 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 7) {
                    updateStateFn.accept("BrickletIO16V2InputPin7",
                            transformBrickletIO16V2InputPin7Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration8 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 8) {
                    updateStateFn.accept("BrickletIO16V2InputPin8",
                            transformBrickletIO16V2InputPin8Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration9 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 9) {
                    updateStateFn.accept("BrickletIO16V2InputPin9",
                            transformBrickletIO16V2InputPin9Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration10 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 10) {
                    updateStateFn.accept("BrickletIO16V2InputPin10",
                            transformBrickletIO16V2InputPin10Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration11 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 11) {
                    updateStateFn.accept("BrickletIO16V2InputPin11",
                            transformBrickletIO16V2InputPin11Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration12 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 12) {
                    updateStateFn.accept("BrickletIO16V2InputPin12",
                            transformBrickletIO16V2InputPin12Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration13 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 13) {
                    updateStateFn.accept("BrickletIO16V2InputPin13",
                            transformBrickletIO16V2InputPin13Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration14 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 14) {
                    updateStateFn.accept("BrickletIO16V2InputPin14",
                            transformBrickletIO16V2InputPin14Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration15 > 1) {
            this.addInputValueListener(this.reg((channel, changed, value) -> {
                if (channel == 15) {
                    updateStateFn.accept("BrickletIO16V2InputPin15",
                            transformBrickletIO16V2InputPin15Callback0(channel, changed, value, cfg));
                }
            }, this::removeInputValueListener));
        }
        if (cfg.pinConfiguration0 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 0) {
                    updateStateFn.accept("BrickletIO16V2OutputPin0",
                            transformBrickletIO16V2OutputPin0Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration1 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 1) {
                    updateStateFn.accept("BrickletIO16V2OutputPin1",
                            transformBrickletIO16V2OutputPin1Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration2 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 2) {
                    updateStateFn.accept("BrickletIO16V2OutputPin2",
                            transformBrickletIO16V2OutputPin2Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration3 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 3) {
                    updateStateFn.accept("BrickletIO16V2OutputPin3",
                            transformBrickletIO16V2OutputPin3Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration4 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 4) {
                    updateStateFn.accept("BrickletIO16V2OutputPin4",
                            transformBrickletIO16V2OutputPin4Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration5 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 5) {
                    updateStateFn.accept("BrickletIO16V2OutputPin5",
                            transformBrickletIO16V2OutputPin5Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration6 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 6) {
                    updateStateFn.accept("BrickletIO16V2OutputPin6",
                            transformBrickletIO16V2OutputPin6Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration7 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 7) {
                    updateStateFn.accept("BrickletIO16V2OutputPin7",
                            transformBrickletIO16V2OutputPin7Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration8 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 8) {
                    updateStateFn.accept("BrickletIO16V2OutputPin8",
                            transformBrickletIO16V2OutputPin8Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration9 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 9) {
                    updateStateFn.accept("BrickletIO16V2OutputPin9",
                            transformBrickletIO16V2OutputPin9Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration10 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 10) {
                    updateStateFn.accept("BrickletIO16V2OutputPin10",
                            transformBrickletIO16V2OutputPin10Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration11 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 11) {
                    updateStateFn.accept("BrickletIO16V2OutputPin11",
                            transformBrickletIO16V2OutputPin11Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration12 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 12) {
                    updateStateFn.accept("BrickletIO16V2OutputPin12",
                            transformBrickletIO16V2OutputPin12Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration13 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 13) {
                    updateStateFn.accept("BrickletIO16V2OutputPin13",
                            transformBrickletIO16V2OutputPin13Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration14 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 14) {
                    updateStateFn.accept("BrickletIO16V2OutputPin14",
                            transformBrickletIO16V2OutputPin14Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }
        if (cfg.pinConfiguration15 <= 1) {
            this.addMonoflopDoneListener(this.reg((channel, value) -> {
                if (channel == 15) {
                    updateStateFn.accept("BrickletIO16V2OutputPin15",
                            transformBrickletIO16V2OutputPin15Callback0(channel, value, cfg));
                }
            }, this::removeMonoflopDoneListener));
        }

        this.setStatusLEDConfig(cfg.statusLEDConfig);
        if (cfg.pinConfiguration0 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin0")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(0, 'i', cfg.pinConfiguration0 % 2 == 1);
            this.setInputValueCallbackConfiguration(0, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration1 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin1")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(1, 'i', cfg.pinConfiguration1 % 2 == 1);
            this.setInputValueCallbackConfiguration(1, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration2 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin2")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(2, 'i', cfg.pinConfiguration2 % 2 == 1);
            this.setInputValueCallbackConfiguration(2, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration3 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin3")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(3, 'i', cfg.pinConfiguration3 % 2 == 1);
            this.setInputValueCallbackConfiguration(3, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration4 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin4")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(4, 'i', cfg.pinConfiguration4 % 2 == 1);
            this.setInputValueCallbackConfiguration(4, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration5 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin5")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(5, 'i', cfg.pinConfiguration5 % 2 == 1);
            this.setInputValueCallbackConfiguration(5, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration6 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin6")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(6, 'i', cfg.pinConfiguration6 % 2 == 1);
            this.setInputValueCallbackConfiguration(6, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration7 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin7")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(7, 'i', cfg.pinConfiguration7 % 2 == 1);
            this.setInputValueCallbackConfiguration(7, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration8 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin8")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(8, 'i', cfg.pinConfiguration8 % 2 == 1);
            this.setInputValueCallbackConfiguration(8, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration9 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin9")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(9, 'i', cfg.pinConfiguration9 % 2 == 1);
            this.setInputValueCallbackConfiguration(9, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration10 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin10")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(10, 'i', cfg.pinConfiguration10 % 2 == 1);
            this.setInputValueCallbackConfiguration(10, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration11 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin11")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(11, 'i', cfg.pinConfiguration11 % 2 == 1);
            this.setInputValueCallbackConfiguration(11, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration12 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin12")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(12, 'i', cfg.pinConfiguration12 % 2 == 1);
            this.setInputValueCallbackConfiguration(12, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration13 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin13")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(13, 'i', cfg.pinConfiguration13 % 2 == 1);
            this.setInputValueCallbackConfiguration(13, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration14 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin14")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(14, 'i', cfg.pinConfiguration14 % 2 == 1);
            this.setInputValueCallbackConfiguration(14, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration15 > 1) {
            BrickletIO16V2InputValueConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2InputPin15")
                    .as(BrickletIO16V2InputValueConfig.class);
            this.setConfiguration(15, 'i', cfg.pinConfiguration15 % 2 == 1);
            this.setInputValueCallbackConfiguration(15, channelCfg.updateInterval, false);
        }
        if (cfg.pinConfiguration0 <= 1) {
            this.setConfiguration(0, 'o', cfg.pinConfiguration0 % 2 == 1);
        }
        if (cfg.pinConfiguration1 <= 1) {
            this.setConfiguration(1, 'o', cfg.pinConfiguration1 % 2 == 1);
        }
        if (cfg.pinConfiguration2 <= 1) {
            this.setConfiguration(2, 'o', cfg.pinConfiguration2 % 2 == 1);
        }
        if (cfg.pinConfiguration3 <= 1) {
            this.setConfiguration(3, 'o', cfg.pinConfiguration3 % 2 == 1);
        }
        if (cfg.pinConfiguration4 <= 1) {
            this.setConfiguration(4, 'o', cfg.pinConfiguration4 % 2 == 1);
        }
        if (cfg.pinConfiguration5 <= 1) {
            this.setConfiguration(5, 'o', cfg.pinConfiguration5 % 2 == 1);
        }
        if (cfg.pinConfiguration6 <= 1) {
            this.setConfiguration(6, 'o', cfg.pinConfiguration6 % 2 == 1);
        }
        if (cfg.pinConfiguration7 <= 1) {
            this.setConfiguration(7, 'o', cfg.pinConfiguration7 % 2 == 1);
        }
        if (cfg.pinConfiguration8 <= 1) {
            this.setConfiguration(8, 'o', cfg.pinConfiguration8 % 2 == 1);
        }
        if (cfg.pinConfiguration9 <= 1) {
            this.setConfiguration(9, 'o', cfg.pinConfiguration9 % 2 == 1);
        }
        if (cfg.pinConfiguration10 <= 1) {
            this.setConfiguration(10, 'o', cfg.pinConfiguration10 % 2 == 1);
        }
        if (cfg.pinConfiguration11 <= 1) {
            this.setConfiguration(11, 'o', cfg.pinConfiguration11 % 2 == 1);
        }
        if (cfg.pinConfiguration12 <= 1) {
            this.setConfiguration(12, 'o', cfg.pinConfiguration12 % 2 == 1);
        }
        if (cfg.pinConfiguration13 <= 1) {
            this.setConfiguration(13, 'o', cfg.pinConfiguration13 % 2 == 1);
        }
        if (cfg.pinConfiguration14 <= 1) {
            this.setConfiguration(14, 'o', cfg.pinConfiguration14 % 2 == 1);
        }
        if (cfg.pinConfiguration15 <= 1) {
            this.setConfiguration(15, 'o', cfg.pinConfiguration15 % 2 == 1);
        }
        if (cfg.pinConfiguration0 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin0")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(0, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin0"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration1 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin1")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(1, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin1"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration2 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin2")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(2, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin2"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration3 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin3")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(3, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin3"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration4 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin4")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(4, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin4"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration5 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin5")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(5, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin5"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration6 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin6")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(6, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin6"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration7 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin7")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(7, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin7"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration8 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin8")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(8, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin8"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration9 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin9")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(9, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin9"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration10 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin10")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(10, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin10"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration11 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin11")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(11, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin11"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration12 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin12")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(12, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin12"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration13 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin13")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(13, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin13"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration14 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin14")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(14, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin14"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
        if (cfg.pinConfiguration15 > 1) {
            BrickletIO16V2EdgeCountConfig channelCfg = getChannelConfigFn.apply("BrickletIO16V2EdgeCountPin15")
                    .as(BrickletIO16V2EdgeCountConfig.class);
            this.setEdgeCountConfiguration(15, channelCfg.edgeType, channelCfg.debounce);
            if (channelCfg.updateInterval > 0) {
                this.manualChannelUpdates.add(scheduler.scheduleWithFixedDelay(() -> {
                    if (handler.getThing().getStatus() == ThingStatus.ONLINE) {
                        handler.handleCommand(
                                new ChannelUID(handler.getThing().getUID(), "BrickletIO16V2EdgeCountPin15"),
                                RefreshType.REFRESH);
                    }
                }, channelCfg.updateInterval, channelCfg.updateInterval, TimeUnit.MILLISECONDS));
            }
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        BrickletIO16V2Config cfg = (BrickletIO16V2Config) config.as(BrickletIO16V2Config.class);
        if (cfg.pinConfiguration0 > 1) {
            this.setInputValueCallbackConfiguration(0, 0, false);
        }
        if (cfg.pinConfiguration1 > 1) {
            this.setInputValueCallbackConfiguration(1, 0, false);
        }
        if (cfg.pinConfiguration2 > 1) {
            this.setInputValueCallbackConfiguration(2, 0, false);
        }
        if (cfg.pinConfiguration3 > 1) {
            this.setInputValueCallbackConfiguration(3, 0, false);
        }
        if (cfg.pinConfiguration4 > 1) {
            this.setInputValueCallbackConfiguration(4, 0, false);
        }
        if (cfg.pinConfiguration5 > 1) {
            this.setInputValueCallbackConfiguration(5, 0, false);
        }
        if (cfg.pinConfiguration6 > 1) {
            this.setInputValueCallbackConfiguration(6, 0, false);
        }
        if (cfg.pinConfiguration7 > 1) {
            this.setInputValueCallbackConfiguration(7, 0, false);
        }
        if (cfg.pinConfiguration8 > 1) {
            this.setInputValueCallbackConfiguration(8, 0, false);
        }
        if (cfg.pinConfiguration9 > 1) {
            this.setInputValueCallbackConfiguration(9, 0, false);
        }
        if (cfg.pinConfiguration10 > 1) {
            this.setInputValueCallbackConfiguration(10, 0, false);
        }
        if (cfg.pinConfiguration11 > 1) {
            this.setInputValueCallbackConfiguration(11, 0, false);
        }
        if (cfg.pinConfiguration12 > 1) {
            this.setInputValueCallbackConfiguration(12, 0, false);
        }
        if (cfg.pinConfiguration13 > 1) {
            this.setInputValueCallbackConfiguration(13, 0, false);
        }
        if (cfg.pinConfiguration14 > 1) {
            this.setInputValueCallbackConfiguration(14, 0, false);
        }
        if (cfg.pinConfiguration15 > 1) {
            this.setInputValueCallbackConfiguration(15, 0, false);
        }
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        BrickletIO16V2Config cfg = (BrickletIO16V2Config) config.as(BrickletIO16V2Config.class);
        List<String> result = new ArrayList<String>();
        if (cfg.pinConfiguration0 > 1) {
            result.add("BrickletIO16V2InputPin0");
        }
        if (cfg.pinConfiguration1 > 1) {
            result.add("BrickletIO16V2InputPin1");
        }
        if (cfg.pinConfiguration2 > 1) {
            result.add("BrickletIO16V2InputPin2");
        }
        if (cfg.pinConfiguration3 > 1) {
            result.add("BrickletIO16V2InputPin3");
        }
        if (cfg.pinConfiguration4 > 1) {
            result.add("BrickletIO16V2InputPin4");
        }
        if (cfg.pinConfiguration5 > 1) {
            result.add("BrickletIO16V2InputPin5");
        }
        if (cfg.pinConfiguration6 > 1) {
            result.add("BrickletIO16V2InputPin6");
        }
        if (cfg.pinConfiguration7 > 1) {
            result.add("BrickletIO16V2InputPin7");
        }
        if (cfg.pinConfiguration8 > 1) {
            result.add("BrickletIO16V2InputPin8");
        }
        if (cfg.pinConfiguration9 > 1) {
            result.add("BrickletIO16V2InputPin9");
        }
        if (cfg.pinConfiguration10 > 1) {
            result.add("BrickletIO16V2InputPin10");
        }
        if (cfg.pinConfiguration11 > 1) {
            result.add("BrickletIO16V2InputPin11");
        }
        if (cfg.pinConfiguration12 > 1) {
            result.add("BrickletIO16V2InputPin12");
        }
        if (cfg.pinConfiguration13 > 1) {
            result.add("BrickletIO16V2InputPin13");
        }
        if (cfg.pinConfiguration14 > 1) {
            result.add("BrickletIO16V2InputPin14");
        }
        if (cfg.pinConfiguration15 > 1) {
            result.add("BrickletIO16V2InputPin15");
        }
        if (cfg.pinConfiguration0 <= 1) {
            result.add("BrickletIO16V2OutputPin0");
        }
        if (cfg.pinConfiguration1 <= 1) {
            result.add("BrickletIO16V2OutputPin1");
        }
        if (cfg.pinConfiguration2 <= 1) {
            result.add("BrickletIO16V2OutputPin2");
        }
        if (cfg.pinConfiguration3 <= 1) {
            result.add("BrickletIO16V2OutputPin3");
        }
        if (cfg.pinConfiguration4 <= 1) {
            result.add("BrickletIO16V2OutputPin4");
        }
        if (cfg.pinConfiguration5 <= 1) {
            result.add("BrickletIO16V2OutputPin5");
        }
        if (cfg.pinConfiguration6 <= 1) {
            result.add("BrickletIO16V2OutputPin6");
        }
        if (cfg.pinConfiguration7 <= 1) {
            result.add("BrickletIO16V2OutputPin7");
        }
        if (cfg.pinConfiguration8 <= 1) {
            result.add("BrickletIO16V2OutputPin8");
        }
        if (cfg.pinConfiguration9 <= 1) {
            result.add("BrickletIO16V2OutputPin9");
        }
        if (cfg.pinConfiguration10 <= 1) {
            result.add("BrickletIO16V2OutputPin10");
        }
        if (cfg.pinConfiguration11 <= 1) {
            result.add("BrickletIO16V2OutputPin11");
        }
        if (cfg.pinConfiguration12 <= 1) {
            result.add("BrickletIO16V2OutputPin12");
        }
        if (cfg.pinConfiguration13 <= 1) {
            result.add("BrickletIO16V2OutputPin13");
        }
        if (cfg.pinConfiguration14 <= 1) {
            result.add("BrickletIO16V2OutputPin14");
        }
        if (cfg.pinConfiguration15 <= 1) {
            result.add("BrickletIO16V2OutputPin15");
        }
        if (cfg.pinConfiguration0 <= 1) {
            result.add("BrickletIO16V2MonoflopPin0");
        }
        if (cfg.pinConfiguration1 <= 1) {
            result.add("BrickletIO16V2MonoflopPin1");
        }
        if (cfg.pinConfiguration2 <= 1) {
            result.add("BrickletIO16V2MonoflopPin2");
        }
        if (cfg.pinConfiguration3 <= 1) {
            result.add("BrickletIO16V2MonoflopPin3");
        }
        if (cfg.pinConfiguration4 <= 1) {
            result.add("BrickletIO16V2MonoflopPin4");
        }
        if (cfg.pinConfiguration5 <= 1) {
            result.add("BrickletIO16V2MonoflopPin5");
        }
        if (cfg.pinConfiguration6 <= 1) {
            result.add("BrickletIO16V2MonoflopPin6");
        }
        if (cfg.pinConfiguration7 <= 1) {
            result.add("BrickletIO16V2MonoflopPin7");
        }
        if (cfg.pinConfiguration8 <= 1) {
            result.add("BrickletIO16V2MonoflopPin8");
        }
        if (cfg.pinConfiguration9 <= 1) {
            result.add("BrickletIO16V2MonoflopPin9");
        }
        if (cfg.pinConfiguration10 <= 1) {
            result.add("BrickletIO16V2MonoflopPin10");
        }
        if (cfg.pinConfiguration11 <= 1) {
            result.add("BrickletIO16V2MonoflopPin11");
        }
        if (cfg.pinConfiguration12 <= 1) {
            result.add("BrickletIO16V2MonoflopPin12");
        }
        if (cfg.pinConfiguration13 <= 1) {
            result.add("BrickletIO16V2MonoflopPin13");
        }
        if (cfg.pinConfiguration14 <= 1) {
            result.add("BrickletIO16V2MonoflopPin14");
        }
        if (cfg.pinConfiguration15 <= 1) {
            result.add("BrickletIO16V2MonoflopPin15");
        }
        if (cfg.pinConfiguration0 > 1) {
            result.add("BrickletIO16V2EdgeCountPin0");
        }
        if (cfg.pinConfiguration1 > 1) {
            result.add("BrickletIO16V2EdgeCountPin1");
        }
        if (cfg.pinConfiguration2 > 1) {
            result.add("BrickletIO16V2EdgeCountPin2");
        }
        if (cfg.pinConfiguration3 > 1) {
            result.add("BrickletIO16V2EdgeCountPin3");
        }
        if (cfg.pinConfiguration4 > 1) {
            result.add("BrickletIO16V2EdgeCountPin4");
        }
        if (cfg.pinConfiguration5 > 1) {
            result.add("BrickletIO16V2EdgeCountPin5");
        }
        if (cfg.pinConfiguration6 > 1) {
            result.add("BrickletIO16V2EdgeCountPin6");
        }
        if (cfg.pinConfiguration7 > 1) {
            result.add("BrickletIO16V2EdgeCountPin7");
        }
        if (cfg.pinConfiguration8 > 1) {
            result.add("BrickletIO16V2EdgeCountPin8");
        }
        if (cfg.pinConfiguration9 > 1) {
            result.add("BrickletIO16V2EdgeCountPin9");
        }
        if (cfg.pinConfiguration10 > 1) {
            result.add("BrickletIO16V2EdgeCountPin10");
        }
        if (cfg.pinConfiguration11 > 1) {
            result.add("BrickletIO16V2EdgeCountPin11");
        }
        if (cfg.pinConfiguration12 > 1) {
            result.add("BrickletIO16V2EdgeCountPin12");
        }
        if (cfg.pinConfiguration13 > 1) {
            result.add("BrickletIO16V2EdgeCountPin13");
        }
        if (cfg.pinConfiguration14 > 1) {
            result.add("BrickletIO16V2EdgeCountPin14");
        }
        if (cfg.pinConfiguration15 > 1) {
            result.add("BrickletIO16V2EdgeCountPin15");
        }
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletIO16V2InputValue":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"), "Input Value", "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIO16V2InputValue"))
                        .withDescription("The logic level that is currently measured on the pin.").build();
            case "BrickletIO16V2OutputPin":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"), "Output Value", "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIO16V2OutputPin"))
                        .withDescription("The logic level that is currently set on the pin.").build();
            case "BrickletIO16V2Monoflop":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"), "NOT USED", "String")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIO16V2Monoflop"))
                        .withDescription("Triggers a monoflop as configured")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("TRIGGER", "Trigger")).build())
                        .build();
            case "BrickletIO16V2EdgeCount":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"), "Edge Count",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIO16V2EdgeCount"))
                        .withDescription("The current value of the edge counter for the selected channel")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge IO-16 Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("16-channel digital input/output")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 0/A0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 1/A1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 2/A2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 3/A3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin4",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 4/A4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin5",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 5/A5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin6",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 6/A6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin7",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 7/A7").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin8",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 8/B0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin9",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 9/B1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin10",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 10/B2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin11",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 11/B3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin12",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 12/B4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin13",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 13/B5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin14",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 14/B6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2InputPin15",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2InputValue"))
                                        .withLabel("Input Value Pin 15/B7").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 0/A0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 1/A1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 2/A2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 3/A3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin4",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 4/A4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin5",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 5/A5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin6",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 6/A6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin7",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 7/A7").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin8",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 8/B0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin9",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 9/B1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin10",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 10/B2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin11",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 11/B3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin12",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 12/B4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin13",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 13/B5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin14",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 14/B6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2OutputPin15",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2OutputPin"))
                                        .withLabel("Output Value Pin 15/B7").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 0/A0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 1/A1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 2/A2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 3/A3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin4",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 4/A4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin5",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 5/A5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin6",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 6/A6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin7",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 7/A7").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin8",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 8/B0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin9",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 9/B1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin10",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 10/B2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin11",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 11/B3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin12",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 12/B4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin13",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 13/B5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin14",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 14/B6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2MonoflopPin15",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2Monoflop"))
                                        .withLabel("Monoflop Pin 15/B7").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 0/A0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 1/A1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 2/A2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 3/A3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin4",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 4/A4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin5",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 5/A5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin6",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 6/A6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin7",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 7/A7").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin8",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 8/B0").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin9",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 9/B1").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin10",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 10/B2").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin11",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 11/B3").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin12",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 12/B4").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin13",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 13/B5").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin14",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 14/B6").build(),
                        new ChannelDefinitionBuilder("BrickletIO16V2EdgeCountPin15",
                                new ChannelTypeUID("tinkerforge", "BrickletIO16V2EdgeCount"))
                                        .withLabel("Edge Count Pin 15/B7").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletio16v2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("pinConfiguration0", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 0/A0. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 0/A0").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration1", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 1/A1. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 1/A1").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration2", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 2/A2. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 2/A2").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration3", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 3/A3. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 3/A3").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration4", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 4/A4. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 4/A4").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration5", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 5/A5. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 5/A5").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration6", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 6/A6. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 6/A6").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration7", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 7/A7. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 7/A7").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration8", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 8/B0. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 8/B0").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration9", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 9/B1. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 9/B1").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration10", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 10/B2. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 10/B2").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration11", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 11/B3. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 11/B3").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration12", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 12/B4. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 12/B4").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration13", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 13/B5. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 13/B5").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration14", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 14/B6. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 14/B6").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("pinConfiguration15", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "Configures the direction of pin 15/B7. Inputs without pull-up will be floating if nothing is connected. Outputs can have an initial state of low or high.")
                                .withLabel("Pin Configuration 15/B7").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("3", "Input with pull-up"),
                                        new ParameterOption("2", "Input without pull-up"),
                                        new ParameterOption("1", "Output (Initial high)"),
                                        new ParameterOption("0", "Output (Initial low)")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIO16V2InputValue":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIO16V2OutputPin":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIO16V2Monoflop":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("monoflopDuration", Type.INTEGER).withDefault("1000")
                                .withDescription("The time (in ms) that the pin should hold the configured value.")
                                .withLabel("Monoflop Duration").withUnit("ms").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("monoflopValue", Type.BOOLEAN).withDefault("true")
                                .withDescription(
                                        "The desired value of the specified channel. Activated means relay closed and Deactivated means relay open.")
                                .withLabel("Monoflop Value").withAdvanced(true).build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIO16V2EdgeCount":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder
                                .create("edgeType", Type.INTEGER).withDefault("0")
                                .withDescription(
                                        "The edge type parameter configures if rising edges, falling edges or both are counted.")
                                .withLabel("Edge Type").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(
                                        Arrays.asList(
                                                new ParameterOption("0", "Rising"), new ParameterOption("1", "Falling"),
                                                new ParameterOption("2", "Both")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("debounce", Type.INTEGER).withDefault("100")
                                .withDescription("The debounce time in ms.").withLabel("Debounce Time")
                                .withAdvanced(true).withMinimum(BigDecimal.valueOf(0))
                                .withMaximum(BigDecimal.valueOf(255)).build(),
                        ConfigDescriptionParameterBuilder.create("resetOnRead", Type.BOOLEAN).withDefault("false")
                                .withDescription(
                                        "Enabling this will reset the edge counter after OpenHAB reads its value. Use this if you want relative edge counts per update.")
                                .withLabel("Reset Edge Count On Update").withAdvanced(true).build(),
                        ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withLabel("Update Interval").withUnit("ms").build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIO16V2Config cfg = (BrickletIO16V2Config) config.as(BrickletIO16V2Config.class);
        switch (channel) {
            case "BrickletIO16V2InputPin0": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin0Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin1": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin1Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin2": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin2Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin3": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin3Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin4": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin4Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin5": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin5Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin6": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin6Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin7": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin7Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin8": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin8Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin9": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin9Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin10": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin10Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin11": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin11Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin12": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin12Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin13": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin13Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin14": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin14Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2InputPin15": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2InputPin15Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin0": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin0Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin1": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin1Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin2": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin2Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin3": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin3Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin4": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin4Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin5": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin5Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin6": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin6Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin7": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin7Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin8": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin8Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin9": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin9Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin10": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin10Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin11": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin11Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin12": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin12Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin13": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin13Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin14": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin14Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2OutputPin15": {
                boolean[] value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2OutputPin15Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin0": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(0);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin0Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin1": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(1);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin1Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin2": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(2);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin2Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin3": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(3);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin3Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin4": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(4);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin4Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin5": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(5);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin5Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin6": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(6);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin6Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin7": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(7);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin7Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin8": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(8);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin8Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin9": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(9);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin9Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin10": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(10);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin10Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin11": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(11);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin11Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin12": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(12);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin12Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin13": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(13);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin13Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin14": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(14);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin14Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2MonoflopPin15": {
                BrickletIO16V2.Monoflop value = this.getMonoflop(15);
                if (true)
                    updateStateFn.accept(channel, transformBrickletIO16V2MonoflopPin15Getter0(value, cfg));
            }
                return;
            case "BrickletIO16V2EdgeCountPin0": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(0, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin0Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin1": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(1, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin1Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin2": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(2, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin2Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin3": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(3, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin3Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin4": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(4, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin4Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin5": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(5, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin5Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin6": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(6, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin6Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin7": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(7, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin7Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin8": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(8, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin8Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin9": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(9, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin9Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin10": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(10, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin10Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin11": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(11, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin11Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin12": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(12, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin12Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin13": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(13, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin13Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin14": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(14, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin14Getter0(value, cfg));
                }
                return;
            }
            case "BrickletIO16V2EdgeCountPin15": {
                BrickletIO16V2EdgeCountConfig channelCfg = channelConfig.as(BrickletIO16V2EdgeCountConfig.class);
                {
                    long value = this.getEdgeCount(15, channelCfg.resetOnRead);
                    if (true)
                        updateStateFn.accept(channel, transformBrickletIO16V2EdgeCountPin15Getter0(value, cfg));
                }
                return;
            }
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletIO16V2OutputPin0":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(0, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin1":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(1, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin2":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(2, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin3":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(3, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin4":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(4, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin5":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(5, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin6":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(6, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin7":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(7, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin8":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(8, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin9":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(9, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin10":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(10, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin11":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(11, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin12":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(12, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin13":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(13, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin14":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(14, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2OutputPin15":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(15, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIO16V2MonoflopPin0":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(0, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin0", 0));
                break;
            case "BrickletIO16V2MonoflopPin1":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(1, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin1", 0));
                break;
            case "BrickletIO16V2MonoflopPin2":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(2, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin2", 0));
                break;
            case "BrickletIO16V2MonoflopPin3":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(3, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin3", 0));
                break;
            case "BrickletIO16V2MonoflopPin4":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(4, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin4", 0));
                break;
            case "BrickletIO16V2MonoflopPin5":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(5, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin5", 0));
                break;
            case "BrickletIO16V2MonoflopPin6":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(6, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin6", 0));
                break;
            case "BrickletIO16V2MonoflopPin7":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(7, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin7", 0));
                break;
            case "BrickletIO16V2MonoflopPin8":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(8, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin8", 0));
                break;
            case "BrickletIO16V2MonoflopPin9":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(9, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin9", 0));
                break;
            case "BrickletIO16V2MonoflopPin10":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(10, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin10", 0));
                break;
            case "BrickletIO16V2MonoflopPin11":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(11, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin11", 0));
                break;
            case "BrickletIO16V2MonoflopPin12":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(12, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin12", 0));
                break;
            case "BrickletIO16V2MonoflopPin13":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(13, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin13", 0));
                break;
            case "BrickletIO16V2MonoflopPin14":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(14, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin14", 0));
                break;
            case "BrickletIO16V2MonoflopPin15":
                if (command instanceof StringType) {
                    BrickletIO16V2MonoflopConfig channelCfg = channelConfig.as(BrickletIO16V2MonoflopConfig.class);
                    this.setMonoflop(15, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIO16V2OutputPin15", 0));
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin0Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin1Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin2Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin3Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin4Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin5Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin6Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin7Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin8Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin9Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin10Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin11Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin12Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin13Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin14Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin15Callback0(int channel,
            boolean changed, boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin0Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin1Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin2Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin3Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin4Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin5Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin6Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin7Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin8Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin9Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin10Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin11Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin12Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin13Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin14Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin15Callback0(int channel,
            boolean value, BrickletIO16V2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin0Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[0] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin1Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[1] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin2Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[2] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin3Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[3] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin4Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[4] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin5Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[5] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin6Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[6] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin7Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[7] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin8Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[8] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin9Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[9] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin10Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[10] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin11Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[11] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin12Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[12] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin13Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[13] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin14Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[14] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2InputPin15Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[15] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin0Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[0] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin1Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[1] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin2Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[2] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin3Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[3] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin4Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[4] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin5Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[5] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin6Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[6] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin7Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[7] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin8Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[8] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin9Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[9] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin10Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[10] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin11Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[11] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin12Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[12] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin13Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[13] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin14Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[14] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2OutputPin15Getter0(boolean[] value,
            BrickletIO16V2Config cfg) {
        return value[15] ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin0Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin1Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin2Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin3Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin4Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin5Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin6Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin7Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin8Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin9Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin10Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin11Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin12Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin13Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin14Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2MonoflopPin15Getter0(
            BrickletIO16V2.Monoflop value, BrickletIO16V2Config cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin0Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin1Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin2Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin3Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin4Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin5Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin6Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin7Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin8Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin9Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin10Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin11Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin12Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin13Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin14Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIO16V2EdgeCountPin15Getter0(long value,
            BrickletIO16V2Config cfg) {
        return new DecimalType(value);
    }
}
