/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletIndustrialDigitalOut4;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Industrial Digital Out 4 Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialDigitalOut4Wrapper extends BrickletIndustrialDigitalOut4
        implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialDigitalOut4.DEVICE_DISPLAY_NAME,
            "brickletindustrialdigitalout4", BrickletIndustrialDigitalOut4.DEVICE_IDENTIFIER,
            BrickletIndustrialDigitalOut4Wrapper.class, BrickletIndustrialDigitalOut4Actions.class, "2.0.0", false);

    public BrickletIndustrialDigitalOut4Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialDigitalOut4Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialDigitalOut4Config cfg = (BrickletIndustrialDigitalOut4Config) config
                .as(BrickletIndustrialDigitalOut4Config.class);
        this.addMonoflopDoneListener(this.reg((selectionMask, valueMask) -> {
            if ((selectionMask & (1 << 0)) > 0) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4OutputPin0",
                        transformBrickletIndustrialDigitalOut4OutputPin0Callback0(selectionMask, valueMask, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((selectionMask, valueMask) -> {
            if ((selectionMask & (1 << 1)) > 0) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4OutputPin1",
                        transformBrickletIndustrialDigitalOut4OutputPin1Callback0(selectionMask, valueMask, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((selectionMask, valueMask) -> {
            if ((selectionMask & (1 << 2)) > 0) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4OutputPin2",
                        transformBrickletIndustrialDigitalOut4OutputPin2Callback0(selectionMask, valueMask, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((selectionMask, valueMask) -> {
            if ((selectionMask & (1 << 3)) > 0) {
                updateStateFn.accept("BrickletIndustrialDigitalOut4OutputPin3",
                        transformBrickletIndustrialDigitalOut4OutputPin3Callback0(selectionMask, valueMask, cfg));
            }
        }, this::removeMonoflopDoneListener));
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialDigitalOut4OutputPin0");
        result.add("BrickletIndustrialDigitalOut4OutputPin1");
        result.add("BrickletIndustrialDigitalOut4OutputPin2");
        result.add("BrickletIndustrialDigitalOut4OutputPin3");
        result.add("BrickletIndustrialDigitalOut4MonoflopPin0");
        result.add("BrickletIndustrialDigitalOut4MonoflopPin1");
        result.add("BrickletIndustrialDigitalOut4MonoflopPin2");
        result.add("BrickletIndustrialDigitalOut4MonoflopPin3");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletIndustrialDigitalOut4OutputPin":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4OutputPin"),
                                "Output Value", "Switch")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialDigitalOut4OutputPin"))
                        .withDescription("The logic level that is currently set on the pin.").build();
            case "BrickletIndustrialDigitalOut4Monoflop":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4Monoflop"), "NOT USED",
                                "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialDigitalOut4Monoflop"))
                        .withDescription("Triggers a monoflop as configured")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("TRIGGER", "Trigger")).build())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Digital Out 4 Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("4 galvanically isolated digital outputs")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4OutputPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4OutputPin"))
                                        .withLabel("Output Value (Pin 0)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4OutputPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4OutputPin"))
                                        .withLabel("Output Value (Pin 1)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4OutputPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4OutputPin"))
                                        .withLabel("Output Value (Pin 2)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4OutputPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4OutputPin"))
                                        .withLabel("Output Value (Pin 3)").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4MonoflopPin0",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4Monoflop"))
                                        .withLabel("Monoflop Pin 0").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4MonoflopPin1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4Monoflop"))
                                        .withLabel("Monoflop Pin 1").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4MonoflopPin2",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4Monoflop"))
                                        .withLabel("Monoflop Pin 2").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDigitalOut4MonoflopPin3",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDigitalOut4Monoflop"))
                                        .withLabel("Monoflop Pin 3").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialdigitalout4":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDigitalOut4OutputPin":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDigitalOut4Monoflop":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("monoflopDuration", Type.INTEGER).withDefault("1000")
                                .withDescription("The time (in ms) that the pin should hold the configured value.")
                                .withLabel("Monoflop Duration").withUnit("ms").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("monoflopValue", Type.BOOLEAN).withDefault("true")
                                .withDescription(
                                        "The desired value of the specified channel. Activated means relay closed and Deactivated means relay open.")
                                .withLabel("Monoflop Value").withAdvanced(true).build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialDigitalOut4Config cfg = (BrickletIndustrialDigitalOut4Config) config
                .as(BrickletIndustrialDigitalOut4Config.class);
        switch (channel) {
            case "BrickletIndustrialDigitalOut4OutputPin0": {
                int value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialDigitalOut4OutputPin0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4OutputPin1": {
                int value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialDigitalOut4OutputPin1Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4OutputPin2": {
                int value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialDigitalOut4OutputPin2Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4OutputPin3": {
                int value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialDigitalOut4OutputPin3Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4MonoflopPin0": {
                BrickletIndustrialDigitalOut4.Monoflop value = this.getMonoflop((short) 0);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4MonoflopPin0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4MonoflopPin1": {
                BrickletIndustrialDigitalOut4.Monoflop value = this.getMonoflop((short) 1);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4MonoflopPin1Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4MonoflopPin2": {
                BrickletIndustrialDigitalOut4.Monoflop value = this.getMonoflop((short) 2);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4MonoflopPin2Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDigitalOut4MonoflopPin3": {
                BrickletIndustrialDigitalOut4.Monoflop value = this.getMonoflop((short) 3);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDigitalOut4MonoflopPin3Getter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletIndustrialDigitalOut4OutputPin0":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValues((short) (1 << 0), cmd == OnOffType.ON ? (short) 0xFF : (short) 0);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4OutputPin1":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValues((short) (1 << 1), cmd == OnOffType.ON ? (short) 0xFF : (short) 0);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4OutputPin2":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValues((short) (1 << 2), cmd == OnOffType.ON ? (short) 0xFF : (short) 0);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4OutputPin3":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValues((short) (1 << 3), cmd == OnOffType.ON ? (short) 0xFF : (short) 0);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDigitalOut4MonoflopPin0":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4MonoflopConfig.class);
                    this.setMonoflop((short) (1 << 0),
                            channelCfg.monoflopValue.booleanValue() ? (short) 0xFF : (short) 0,
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4OutputPin0", 0));
                break;
            case "BrickletIndustrialDigitalOut4MonoflopPin1":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4MonoflopConfig.class);
                    this.setMonoflop((short) (1 << 1),
                            channelCfg.monoflopValue.booleanValue() ? (short) 0xFF : (short) 0,
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4OutputPin1", 0));
                break;
            case "BrickletIndustrialDigitalOut4MonoflopPin2":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4MonoflopConfig.class);
                    this.setMonoflop((short) (1 << 2),
                            channelCfg.monoflopValue.booleanValue() ? (short) 0xFF : (short) 0,
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4OutputPin2", 0));
                break;
            case "BrickletIndustrialDigitalOut4MonoflopPin3":
                if (command instanceof StringType) {
                    BrickletIndustrialDigitalOut4MonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDigitalOut4MonoflopConfig.class);
                    this.setMonoflop((short) (1 << 3),
                            channelCfg.monoflopValue.booleanValue() ? (short) 0xFF : (short) 0,
                            channelCfg.monoflopDuration.longValue());
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDigitalOut4OutputPin3", 0));
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin0Callback0(
            int selectionMask, int valueMask, BrickletIndustrialDigitalOut4Config cfg) {
        return (valueMask & (1 << 0)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin1Callback0(
            int selectionMask, int valueMask, BrickletIndustrialDigitalOut4Config cfg) {
        return (valueMask & (1 << 1)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin2Callback0(
            int selectionMask, int valueMask, BrickletIndustrialDigitalOut4Config cfg) {
        return (valueMask & (1 << 2)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin3Callback0(
            int selectionMask, int valueMask, BrickletIndustrialDigitalOut4Config cfg) {
        return (valueMask & (1 << 3)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin0Getter0(int value,
            BrickletIndustrialDigitalOut4Config cfg) {
        return (value & (1 << 0)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin1Getter0(int value,
            BrickletIndustrialDigitalOut4Config cfg) {
        return (value & (1 << 1)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin2Getter0(int value,
            BrickletIndustrialDigitalOut4Config cfg) {
        return (value & (1 << 2)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4OutputPin3Getter0(int value,
            BrickletIndustrialDigitalOut4Config cfg) {
        return (value & (1 << 3)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4MonoflopPin0Getter0(
            BrickletIndustrialDigitalOut4.Monoflop value, BrickletIndustrialDigitalOut4Config cfg) {
        return value.value > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4MonoflopPin1Getter0(
            BrickletIndustrialDigitalOut4.Monoflop value, BrickletIndustrialDigitalOut4Config cfg) {
        return value.value > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4MonoflopPin2Getter0(
            BrickletIndustrialDigitalOut4.Monoflop value, BrickletIndustrialDigitalOut4Config cfg) {
        return value.value > 0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDigitalOut4MonoflopPin3Getter0(
            BrickletIndustrialDigitalOut4.Monoflop value, BrickletIndustrialDigitalOut4Config cfg) {
        return value.value > 0 ? OnOffType.ON : OnOffType.OFF;
    }
}
