/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletIndustrialDualAnalogIn;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Industrial Dual Analog In Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDualAnalogInActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type = "int") Map<String, Object> brickletIndustrialDualAnalogInGetVoltage(
            @ActionInput(name = "channel") short channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialDualAnalogInWrapper dev = ((@Nullable BrickletIndustrialDualAnalogInWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getVoltage(channel);

        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetVoltage(@Nullable ThingActions actions,
            short channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetVoltage(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Sample Rate")
    public @ActionOutput(name = "rate", type = "short") Map<String, Object> brickletIndustrialDualAnalogInGetSampleRate()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialDualAnalogInWrapper dev = ((@Nullable BrickletIndustrialDualAnalogInWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short value = dev.getSampleRate();

        result.put("rate", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetSampleRate(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetSampleRate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type = "int[]") @ActionOutput(name = "gain", type = "int[]") Map<String, Object> brickletIndustrialDualAnalogInGetCalibration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialDualAnalogInWrapper dev = ((@Nullable BrickletIndustrialDualAnalogInWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialDualAnalogIn.Calibration value = dev.getCalibration();

        result.put("offset", value.offset);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetCalibration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get ADC Values")
    public @ActionOutput(name = "value", type = "int[]") Map<String, Object> brickletIndustrialDualAnalogInGetADCValues()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialDualAnalogInWrapper dev = ((@Nullable BrickletIndustrialDualAnalogInWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int[] value = dev.getADCValues();

        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetADCValues(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetADCValues();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletIndustrialDualAnalogInGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletIndustrialDualAnalogInWrapper dev = ((@Nullable BrickletIndustrialDualAnalogInWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletIndustrialDualAnalogIn.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }
}
