/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletIndustrialDualAnalogIn;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Industrial Dual Analog In Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialDualAnalogInWrapper extends BrickletIndustrialDualAnalogIn
        implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialDualAnalogIn.DEVICE_DISPLAY_NAME,
            "brickletindustrialdualanalogin", BrickletIndustrialDualAnalogIn.DEVICE_IDENTIFIER,
            BrickletIndustrialDualAnalogInWrapper.class, BrickletIndustrialDualAnalogInActions.class, "2.0.0", false);

    public BrickletIndustrialDualAnalogInWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialDualAnalogInWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialDualAnalogInConfig cfg = (BrickletIndustrialDualAnalogInConfig) config
                .as(BrickletIndustrialDualAnalogInConfig.class);
        this.addVoltageListener(this.reg((channel, voltage) -> {
            if (channel == 0) {
                updateStateFn.accept("BrickletIndustrialDualAnalogInVoltageChannel0",
                        transformBrickletIndustrialDualAnalogInVoltageChannel0Callback0(channel, voltage, cfg));
            }
        }, this::removeVoltageListener));
        this.addVoltageListener(this.reg((channel, voltage) -> {
            if (channel == 1) {
                updateStateFn.accept("BrickletIndustrialDualAnalogInVoltageChannel1",
                        transformBrickletIndustrialDualAnalogInVoltageChannel1Callback0(channel, voltage, cfg));
            }
        }, this::removeVoltageListener));
        this.setSampleRate(cfg.sampleRate.shortValue());
        {
            BrickletIndustrialDualAnalogInVoltageConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialDualAnalogInVoltageChannel0")
                    .as(BrickletIndustrialDualAnalogInVoltageConfig.class);
            this.setVoltageCallbackPeriod((short) 0, channelCfg.updateInterval);
        }
        {
            BrickletIndustrialDualAnalogInVoltageConfig channelCfg = getChannelConfigFn
                    .apply("BrickletIndustrialDualAnalogInVoltageChannel1")
                    .as(BrickletIndustrialDualAnalogInVoltageConfig.class);
            this.setVoltageCallbackPeriod((short) 1, channelCfg.updateInterval);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setVoltageCallbackPeriod((short) 0, 0);
        this.setVoltageCallbackPeriod((short) 1, 0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialDualAnalogInVoltageChannel0");
        result.add("BrickletIndustrialDualAnalogInVoltageChannel1");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletIndustrialDualAnalogInVoltage":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualAnalogInVoltage"), "NOT USED",
                                "Number:ElectricPotential")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialDualAnalogInVoltage"))
                        .withDescription("Measured voltage between -35 and 35 V")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-35))
                                        .withMaximum(BigDecimal.valueOf(35)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Dual Analog In Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures two DC voltages between -35V and +35V with 24bit resolution each")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletIndustrialDualAnalogInVoltageChannel0",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualAnalogInVoltage"))
                                        .withLabel("Voltage Channel 0").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDualAnalogInVoltageChannel1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualAnalogInVoltage"))
                                        .withLabel("Voltage Channel 1").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialdualanalogin":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("sampleRate", Type.INTEGER).withDefault("6")
                                .withDescription(
                                        "The voltage measurement sample rate. Decreasing the sample rate will also decrease the noise on the data.")
                                .withLabel("Sample Rate").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "976 SPS"),
                                        new ParameterOption("1", "488 SPS"), new ParameterOption("2", "244 SPS"),
                                        new ParameterOption("3", "122 SPS"), new ParameterOption("4", "61 SPS"),
                                        new ParameterOption("5", "4 SPS"), new ParameterOption("6", "2 SPS"),
                                        new ParameterOption("7", "1 SPS")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletIndustrialDualAnalogInVoltage":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialDualAnalogInConfig cfg = (BrickletIndustrialDualAnalogInConfig) config
                .as(BrickletIndustrialDualAnalogInConfig.class);
        switch (channel) {
            case "BrickletIndustrialDualAnalogInVoltageChannel0": {
                int value = this.getVoltage((short) 0);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDualAnalogInVoltageChannel0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDualAnalogInVoltageChannel1": {
                int value = this.getVoltage((short) 1);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDualAnalogInVoltageChannel1Getter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInVoltageChannel0Callback0(
            short channel, int voltage, BrickletIndustrialDualAnalogInConfig cfg) {
        return new QuantityType<>(voltage / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInVoltageChannel1Callback0(
            short channel, int voltage, BrickletIndustrialDualAnalogInConfig cfg) {
        return new QuantityType<>(voltage / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInVoltageChannel0Getter0(
            int value, BrickletIndustrialDualAnalogInConfig cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInVoltageChannel1Getter0(
            int value, BrickletIndustrialDualAnalogInConfig cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }
}
