/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletIndustrialDualRelay;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Industrial Dual Relay Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialDualRelayWrapper extends BrickletIndustrialDualRelay
        implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialDualRelay.DEVICE_DISPLAY_NAME,
            "brickletindustrialdualrelay", BrickletIndustrialDualRelay.DEVICE_IDENTIFIER,
            BrickletIndustrialDualRelayWrapper.class, BrickletIndustrialDualRelayActions.class, "2.0.0", true);

    public BrickletIndustrialDualRelayWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialDualRelayWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialDualRelayConfig cfg = (BrickletIndustrialDualRelayConfig) config
                .as(BrickletIndustrialDualRelayConfig.class);
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 0) {
                updateStateFn.accept("BrickletIndustrialDualRelayRelay0",
                        transformBrickletIndustrialDualRelayRelay0Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));
        this.addMonoflopDoneListener(this.reg((channel, value) -> {
            if (channel == 1) {
                updateStateFn.accept("BrickletIndustrialDualRelayRelay1",
                        transformBrickletIndustrialDualRelayRelay1Callback0(channel, value, cfg));
            }
        }, this::removeMonoflopDoneListener));

        this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialDualRelayRelay0");
        result.add("BrickletIndustrialDualRelayRelay1");
        result.add("BrickletIndustrialDualRelayMonoflopRelay0");
        result.add("BrickletIndustrialDualRelayMonoflopRelay1");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletIndustrialDualRelayRelay":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualRelayRelay"), "NOT USED",
                                "Switch")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialDualRelayRelay"))
                        .withDescription("NOT USED").build();
            case "BrickletIndustrialDualRelayMonoflop":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualRelayMonoflop"), "NOT USED",
                                "String")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletIndustrialDualRelayMonoflop"))
                        .withDescription("Triggers a monoflop as configured")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("TRIGGER", "Trigger")).build())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Dual Relay Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Two relays to switch AC/DC devices")
                .withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletIndustrialDualRelayRelay0",
                        new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualRelayRelay"))
                                .withLabel("Relay 0")
                                .withDescription(
                                        "Switches Relay 0. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.")
                                .build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDualRelayRelay1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualRelayRelay"))
                                        .withLabel("Relay 1")
                                        .withDescription(
                                                "Switches Relay 1. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDualRelayMonoflopRelay0",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualRelayMonoflop"))
                                        .withLabel("Monoflop Relay 0").build(),
                        new ChannelDefinitionBuilder("BrickletIndustrialDualRelayMonoflopRelay1",
                                new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualRelayMonoflop"))
                                        .withLabel("Monoflop Relay 1").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialdualrelay":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletIndustrialDualRelayRelay":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDualRelayMonoflop":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("monoflopDuration", Type.INTEGER).withDefault("1000")
                                .withDescription("The time (in ms) that the relay should hold the configured value.")
                                .withLabel("Monoflop Duration").withUnit("ms").withAdvanced(true)
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("monoflopValue", Type.BOOLEAN).withDefault("true")
                                .withDescription(
                                        "The desired value of the specified channel. Activated means relay closed and Deactivated means relay open.")
                                .withLabel("Monoflop Value").withAdvanced(true).build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialDualRelayConfig cfg = (BrickletIndustrialDualRelayConfig) config
                .as(BrickletIndustrialDualRelayConfig.class);
        switch (channel) {
            case "BrickletIndustrialDualRelayRelay0": {
                BrickletIndustrialDualRelay.Value value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialDualRelayRelay0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDualRelayRelay1": {
                BrickletIndustrialDualRelay.Value value = this.getValue();
                if (true)
                    updateStateFn.accept(channel, transformBrickletIndustrialDualRelayRelay1Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDualRelayMonoflopRelay0": {
                BrickletIndustrialDualRelay.Monoflop value = this.getMonoflop(0);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDualRelayMonoflopRelay0Getter0(value, cfg));
            }
                return;
            case "BrickletIndustrialDualRelayMonoflopRelay1": {
                BrickletIndustrialDualRelay.Monoflop value = this.getMonoflop(1);
                if (true)
                    updateStateFn.accept(channel,
                            transformBrickletIndustrialDualRelayMonoflopRelay1Getter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletIndustrialDualRelayRelay0":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(0, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDualRelayRelay1":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setSelectedValue(1, cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            case "BrickletIndustrialDualRelayMonoflopRelay0":
                if (command instanceof StringType) {
                    BrickletIndustrialDualRelayMonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDualRelayMonoflopConfig.class);
                    this.setMonoflop(0, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDualRelayRelay0", 0));
                break;
            case "BrickletIndustrialDualRelayMonoflopRelay1":
                if (command instanceof StringType) {
                    BrickletIndustrialDualRelayMonoflopConfig channelCfg = channelConfig
                            .as(BrickletIndustrialDualRelayMonoflopConfig.class);
                    this.setMonoflop(1, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDualRelayRelay1", 0));
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualRelayRelay0Callback0(int channel,
            boolean value, BrickletIndustrialDualRelayConfig cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualRelayRelay1Callback0(int channel,
            boolean value, BrickletIndustrialDualRelayConfig cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualRelayRelay0Getter0(
            BrickletIndustrialDualRelay.Value value, BrickletIndustrialDualRelayConfig cfg) {
        return value.channel0 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualRelayRelay1Getter0(
            BrickletIndustrialDualRelay.Value value, BrickletIndustrialDualRelayConfig cfg) {
        return value.channel1 ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualRelayMonoflopRelay0Getter0(
            BrickletIndustrialDualRelay.Monoflop value, BrickletIndustrialDualRelayConfig cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualRelayMonoflopRelay1Getter0(
            BrickletIndustrialDualRelay.Monoflop value, BrickletIndustrialDualRelayConfig cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }
}
