/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletJoystick;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Joystick Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletJoystickActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Position")
    public @ActionOutput(name = "x", type = "short") @ActionOutput(name = "y", type = "short") Map<String, Object> brickletJoystickGetPosition()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletJoystickWrapper dev = ((@Nullable BrickletJoystickWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletJoystick.Position value = dev.getPosition();

        result.put("x", value.x);
        result.put("y", value.y);
        return result;
    }

    public static Map<String, Object> brickletJoystickGetPosition(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickGetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }

    @RuleAction(label = "Is Pressed")
    public @ActionOutput(name = "pressed", type = "boolean") Map<String, Object> brickletJoystickIsPressed()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletJoystickWrapper dev = ((@Nullable BrickletJoystickWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.isPressed();

        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletJoystickIsPressed(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickIsPressed();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "x", type = "int") @ActionOutput(name = "y", type = "int") Map<String, Object> brickletJoystickGetAnalogValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletJoystickWrapper dev = ((@Nullable BrickletJoystickWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletJoystick.AnalogValue value = dev.getAnalogValue();

        result.put("x", value.x);
        result.put("y", value.y);
        return result;
    }

    public static Map<String, Object> brickletJoystickGetAnalogValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletJoystickGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletJoystickWrapper dev = ((@Nullable BrickletJoystickWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletJoystick.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletJoystickGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }
}
