/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletLaserRangeFinderV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Laser Range Finder Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletLaserRangeFinderV2Wrapper extends BrickletLaserRangeFinderV2
        implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletLaserRangeFinderV2.DEVICE_DISPLAY_NAME,
            "brickletlaserrangefinderv2", BrickletLaserRangeFinderV2.DEVICE_IDENTIFIER,
            BrickletLaserRangeFinderV2Wrapper.class, BrickletLaserRangeFinderV2Actions.class, "2.0.0", true);

    public BrickletLaserRangeFinderV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletLaserRangeFinderV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletLaserRangeFinderV2Config cfg = (BrickletLaserRangeFinderV2Config) config
                .as(BrickletLaserRangeFinderV2Config.class);
        this.addDistanceListener(this.reg((distance) -> {
            if (true) {
                updateStateFn.accept("BrickletLaserRangeFinderV2Distance",
                        transformBrickletLaserRangeFinderV2DistanceCallback0(distance, cfg));
            }
        }, this::removeDistanceListener));
        this.addVelocityListener(this.reg((velocity) -> {
            if (true) {
                updateStateFn.accept("BrickletLaserRangeFinderV2Velocity",
                        transformBrickletLaserRangeFinderV2VelocityCallback0(velocity, cfg));
            }
        }, this::removeVelocityListener));

        this.setConfiguration(cfg.acquisitionCount, cfg.enableQuickTermination, cfg.thresholdValue,
                cfg.enableFixedMeasurementFrequency ? cfg.measurementFrequency : 0);
        this.setMovingAverage(cfg.distanceMovingAverageLength, cfg.velocityMovingAverageLength);
        this.setDistanceLEDConfig(cfg.distanceLEDConfig);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletLaserRangeFinderV2DistanceConfig channelCfg = getChannelConfigFn
                    .apply("BrickletLaserRangeFinderV2Distance").as(BrickletLaserRangeFinderV2DistanceConfig.class);
            this.setDistanceCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletLaserRangeFinderV2VelocityConfig channelCfg = getChannelConfigFn
                    .apply("BrickletLaserRangeFinderV2Velocity").as(BrickletLaserRangeFinderV2VelocityConfig.class);
            this.setVelocityCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setDistanceCallbackConfiguration(0, true, 'x', 0, 0);
        this.setVelocityCallbackConfiguration(0, true, 'x', 0, 0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletLaserRangeFinderV2Distance");
        result.add("BrickletLaserRangeFinderV2Velocity");
        result.add("BrickletLaserRangeFinderV2EnableLaser");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletLaserRangeFinderV2Distance":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderV2Distance"), "Distance",
                                "Number:Length")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletLaserRangeFinderV2Distance"))
                        .withDescription(
                                "The measured distance. Sensor hardware version 1 cannot measure distance and velocity at the same time. Therefore, the distance mode has to be enabled. Sensor hardware version 3 can measure distance and velocity at the same time. Also the laser has to be enabled.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(40)).withStep(BigDecimal.valueOf(0.01))
                                        .withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletLaserRangeFinderV2Velocity":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderV2Velocity"), "Velocity",
                                "Number:Speed")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletLaserRangeFinderV2Velocity"))
                        .withDescription(
                                "The measured velocity. Sensor hardware version 1 cannot measure distance and velocity at the same time. Therefore, the velocity mode has to be enabled. Sensor hardware version 3 can measure distance and velocity at the same time, but the velocity measurement only produces stables results if a fixed measurement rateis configured. Also the laser has to be enabled.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-128))
                                        .withMaximum(BigDecimal.valueOf(127)).withStep(BigDecimal.valueOf(0.01))
                                        .withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletLaserRangeFinderV2EnableLaser":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderV2EnableLaser"),
                                "Enable Laser", "Switch")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletLaserRangeFinderV2EnableLaser"))
                        .withDescription("Activates the laser of the LIDAR.").build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Laser Range Finder Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures distance up to 40m with laser light")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletLaserRangeFinderV2Distance",
                                new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderV2Distance"))
                                        .withLabel("Distance").build(),
                        new ChannelDefinitionBuilder("BrickletLaserRangeFinderV2Velocity",
                                new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderV2Velocity"))
                                        .withLabel("Velocity").build(),
                        new ChannelDefinitionBuilder("BrickletLaserRangeFinderV2EnableLaser",
                                new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderV2EnableLaser"))
                                        .withLabel("Enable Laser").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletlaserrangefinderv2":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(
                                ConfigDescriptionParameterBuilder.create("distanceMovingAverageLength", Type.INTEGER)
                                        .withDefault("10")
                                        .withDescription("The length of a moving averaging for the distance.")
                                        .withGroupName("average").withLabel("Distance Moving Average Length")
                                        .withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(
                                                BigDecimal.valueOf(255))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("velocityMovingAverageLength", Type.INTEGER)
                                        .withDefault("10")
                                        .withDescription("The length of a moving averaging for the velocity.")
                                        .withGroupName("average").withLabel("Velocity Moving Average Length")
                                        .withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(
                                                BigDecimal.valueOf(255))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("acquisitionCount", Type.INTEGER)
                                        .withDefault("128")
                                        .withDescription(
                                                "The Acquisition Count defines the number of times the Laser Range Finder Bricklet will integrate acquisitions to find a correlation record peak. With a higher count, the Bricklet can measure longer distances. With a lower count, the rate increases. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.")
                                        .withGroupName("sensor").withLabel("Acquisition Count")
                                        .withMinimum(BigDecimal.valueOf(1)).withMaximum(
                                                BigDecimal.valueOf(255))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("enableQuickTermination", Type.BOOLEAN)
                                        .withDefault("false")
                                        .withDescription(
                                                "If you enable Quick Termination, the distance measurement will be terminated early if a high peak was already detected. This means that a higher measurement rate can be achieved and long distances can be measured at the same time. However, the chance of false-positive distance measurements increases. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.")
                                        .withGroupName("sensor").withLabel("Enable Quick Termination").build(),
                                ConfigDescriptionParameterBuilder.create("thresholdValue", Type.INTEGER)
                                        .withDefault("0")
                                        .withDescription(
                                                "Normally the distance is calculated with a detection algorithm that uses peak value, signal strength and noise. You can however also define a fixed Threshold Value. Set this to a low value if you want to measure the distance to something that has very little reflection (e.g. glass) and set it to a high value if you want to measure the distance to something with a very high reflection (e.g. mirror). Set this to 0 to use the default algorithm. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.")
                                        .withGroupName("sensor").withLabel("Threshold Value")
                                        .withMinimum(BigDecimal.valueOf(0)).withMaximum(
                                                BigDecimal.valueOf(255))
                                        .build(),
                                ConfigDescriptionParameterBuilder
                                        .create("enableFixedMeasurementFrequency", Type.BOOLEAN).withDefault("false")
                                        .withDescription(
                                                "For a stable velocity measurement you should set a fixed measurement frequency. See Measurement Frequency for details. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.")
                                        .withGroupName("sensor").withLabel("Enable Fixed Measurement Frequency")
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("measurementFrequency", Type.INTEGER)
                                        .withDefault("10")
                                        .withDescription(
                                                "Set the Measurement Frequency in Hz to force a fixed measurement rate. If set to 0, the Laser Range Finder Bricklet will use the optimal frequency according to the other configurations and the actual measured distance. Since the rate is not fixed in this case, the velocity measurement is not stable. For a stable velocity measurement you should set a fixed measurement frequency. The lower the frequency, the higher is the resolution of the calculated velocity. The allowed values are 10Hz-500Hz. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.")
                                        .withGroupName("sensor").withLabel("Measurement Frequency")
                                        .withMinimum(BigDecimal.valueOf(10)).withMaximum(BigDecimal.valueOf(500))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("distanceLEDConfig", Type.INTEGER)
                                        .withDefault("3")
                                        .withDescription(
                                                "Configures the distance LED to be either turned off, turned on, blink in heartbeat mode or show the distance (brighter = object is nearer).")
                                        .withLabel("Distance LED Config").withLimitToOptions(true)
                                        .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                                new ParameterOption("1", "On"),
                                                new ParameterOption("2", "Show Heartbeat"),
                                                new ParameterOption("3", "Show Distance")))
                                        .build(),
                                ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER)
                                        .withDefault("3")
                                        .withDescription(
                                                "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                        .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                        .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                                new ParameterOption("1", "On"),
                                                new ParameterOption("2", "Show Heartbeat"),
                                                new ParameterOption("3", "Show Status")))
                                        .build()))
                        .withParameterGroups(Arrays.asList(
                                new ConfigDescriptionParameterGroup("update_intervals", "null", true,
                                        "Update Intervals", "Update Intervals"),
                                new ConfigDescriptionParameterGroup("average", "null", true, "Averaging",
                                        "Sets the different averaging parameters. It is possible to set the length of a normal averaging for the temperature and pressure, as well as an additional length of a moving average for the pressure. The moving average is calculated from the normal averages. There is no moving average for the temperature.<br/><br/>The maximum length for the pressure average is 10, for the temperature average is 255 and for the moving average is 25.<br/><br/>Setting the all three parameters to 0 will turn the averaging completely off. If the averaging is off, there is lots of noise on the data, but the data is without delay. Thus we recommend to turn the averaging off if the Barometer Bricklet data is to be used for sensor fusion with other sensors.<br/><br/>The default values are 10 for the normal averages and 25 for the moving average."),
                                new ConfigDescriptionParameterGroup("sensor3", "null", false, "Sensor Version 3",
                                        "Configuration for LIDAR-Lite sensors with hardware version 3")))
                        .build();
            case "channel-type:tinkerforge:BrickletLaserRangeFinderV2Distance":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLaserRangeFinderV2Velocity":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLaserRangeFinderV2EnableLaser":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletLaserRangeFinderV2Config cfg = (BrickletLaserRangeFinderV2Config) config
                .as(BrickletLaserRangeFinderV2Config.class);
        switch (channel) {
            case "BrickletLaserRangeFinderV2Distance": {
                int value = this.getDistance();
                if (true)
                    updateStateFn.accept(channel, transformBrickletLaserRangeFinderV2DistanceGetter0(value, cfg));
            }
                return;
            case "BrickletLaserRangeFinderV2Velocity": {
                int value = this.getVelocity();
                if (true)
                    updateStateFn.accept(channel, transformBrickletLaserRangeFinderV2VelocityGetter0(value, cfg));
            }
                return;
            case "BrickletLaserRangeFinderV2EnableLaser": {
                boolean value = this.getEnable();
                if (true)
                    updateStateFn.accept(channel, transformBrickletLaserRangeFinderV2EnableLaserGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickletLaserRangeFinderV2EnableLaser":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    this.setEnable(cmd == OnOffType.ON);
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderV2DistanceCallback0(int distance,
            BrickletLaserRangeFinderV2Config cfg) {
        return new QuantityType<>(distance / 100.0, SIUnits.METRE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderV2VelocityCallback0(int velocity,
            BrickletLaserRangeFinderV2Config cfg) {
        return new QuantityType<>(velocity / 100.0, SmartHomeUnits.METRE_PER_SECOND);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderV2DistanceGetter0(int value,
            BrickletLaserRangeFinderV2Config cfg) {
        return new QuantityType<>(value / 100.0, SIUnits.METRE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderV2VelocityGetter0(int value,
            BrickletLaserRangeFinderV2Config cfg) {
        return new QuantityType<>(value / 100.0, SmartHomeUnits.METRE_PER_SECOND);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderV2EnableLaserGetter0(boolean value,
            BrickletLaserRangeFinderV2Config cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }
}
