/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletLoadCellV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Load Cell Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLoadCellV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Weight")
    public @ActionOutput(name = "weight", type = "int") Map<String, Object> brickletLoadCellV2GetWeight()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getWeight();

        result.put("weight", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetWeight(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetWeight();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Info LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletLoadCellV2GetInfoLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getInfoLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetInfoLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetInfoLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Tare")
    public void brickletLoadCellV2Tare() throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.tare();
    }

    public static void brickletLoadCellV2Tare(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2Tare();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type = "int") Map<String, Object> brickletLoadCellV2GetMovingAverage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getMovingAverage();

        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetMovingAverage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "rate", type = "int") @ActionOutput(name = "gain", type = "int") Map<String, Object> brickletLoadCellV2GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLoadCellV2.Configuration value = dev.getConfiguration();

        result.put("rate", value.rate);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletLoadCellV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletLoadCellV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletLoadCellV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletLoadCellV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLoadCellV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletLoadCellV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletLoadCellV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }
}
