/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletLoadCell;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Load Cell Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletLoadCellWrapper extends BrickletLoadCell implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletLoadCell.DEVICE_DISPLAY_NAME,
            "brickletloadcell", BrickletLoadCell.DEVICE_IDENTIFIER, BrickletLoadCellWrapper.class,
            BrickletLoadCellActions.class, "2.0.0", false);

    public BrickletLoadCellWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletLoadCellWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletLoadCellConfig cfg = (BrickletLoadCellConfig) config.as(BrickletLoadCellConfig.class);
        this.addWeightListener(this.reg((weight) -> {
            if (true) {
                updateStateFn.accept("BrickletLoadCellWeight", transformBrickletLoadCellWeightCallback0(weight, cfg));
            }
        }, this::removeWeightListener));
        this.setMovingAverage(cfg.movingAverage.shortValue());
        {
            BrickletLoadCellWeightConfig channelCfg = getChannelConfigFn.apply("BrickletLoadCellWeight")
                    .as(BrickletLoadCellWeightConfig.class);
            this.setWeightCallbackPeriod(channelCfg.updateInterval);
            this.setWeightCallbackThreshold('x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setWeightCallbackPeriod(0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletLoadCellWeight");
        result.add("BrickletLoadCellTare");
        result.add("BrickletLoadCellLED");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletLoadCellWeight":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLoadCellWeight"), "Weight", "Number:Mass")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLoadCellWeight"))
                        .withDescription("The currently measured weight")
                        .withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletLoadCellTare":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLoadCellTare"), "Tare", "String")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLoadCellTare"))
                        .withDescription("Sets the currently measured weight as tare weight.")
                        .withCommandDescription(CommandDescriptionBuilder.create()
                                .withCommandOption(new CommandOption("TARE", "Tare")).build())
                        .build();
            case "BrickletLoadCellLED":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletLoadCellLED"), "LED", "Switch")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLoadCellLED"))
                        .withDescription("Activates/Deactivates the LED.").build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Load Cell Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures weight with a load cell")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletLoadCellWeight",
                                new ChannelTypeUID("tinkerforge", "BrickletLoadCellWeight")).withLabel("Weight")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletLoadCellTare",
                                new ChannelTypeUID("tinkerforge", "BrickletLoadCellTare")).withLabel("Tare").build(),
                        new ChannelDefinitionBuilder("BrickletLoadCellLED",
                                new ChannelTypeUID("tinkerforge", "BrickletLoadCellLED")).withLabel("LED").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletloadcell":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("movingAverage", Type.INTEGER).withDefault("4")
                                .withDescription(
                                        "The length of a moving averaging for the weight value.<br/><br/>Setting the length to 1 will turn the averaging off. With less averaging, there is more noise on the data.")
                                .withLabel("Moving Average").withAdvanced(true).withMinimum(BigDecimal.valueOf(1))
                                .withMaximum(BigDecimal.valueOf(40)).build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletLoadCellWeight":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLoadCellTare":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLoadCellLED":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletLoadCellConfig cfg = (BrickletLoadCellConfig) config.as(BrickletLoadCellConfig.class);
        switch (channel) {
            case "BrickletLoadCellWeight": {
                int value = this.getWeight();
                if (true)
                    updateStateFn.accept(channel, transformBrickletLoadCellWeightGetter0(value, cfg));
            }
                return;
            case "BrickletLoadCellTare":
                return;
            case "BrickletLoadCellLED": {
                boolean value = this.isLEDOn();
                if (true)
                    updateStateFn.accept(channel, transformBrickletLoadCellLEDGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();

        switch (channel) {
            case "BrickletLoadCellTare":
                if (command instanceof StringType) {
                    this.tare();
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.",
                            command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletLoadCellWeight", 0));
                break;
            case "BrickletLoadCellLED":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    if (cmd == OnOffType.ON) {
                        this.ledOn();
                    }
                }

                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
                    if (cmd == OnOffType.OFF) {
                        this.ledOn();
                    }
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType, OnOffType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLoadCellWeightCallback0(int weight,
            BrickletLoadCellConfig cfg) {
        return new QuantityType<>(weight, SIUnits.GRAM);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLoadCellWeightGetter0(int value,
            BrickletLoadCellConfig cfg) {
        return new QuantityType<>(value, SIUnits.GRAM);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletLoadCellLEDGetter0(boolean value,
            BrickletLoadCellConfig cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }
}
