/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletMotorizedLinearPoti;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Motorized Linear Poti Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMotorizedLinearPotiActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Position")
    public @ActionOutput(name = "position", type = "int") Map<String, Object> brickletMotorizedLinearPotiGetPosition()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getPosition();

        result.put("position", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetPosition(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Set Motor Position")
    public void brickletMotorizedLinearPotiSetMotorPosition(@ActionInput(name = "position") int position,
            @ActionInput(name = "driveMode") int driveMode, @ActionInput(name = "holdPosition") boolean holdPosition)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setMotorPosition(position, driveMode, holdPosition);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletMotorizedLinearPotiMotorPosition")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletMotorizedLinearPotiMotorPosition"),
                    RefreshType.REFRESH);
        }
    }

    public static void brickletMotorizedLinearPotiSetMotorPosition(@Nullable ThingActions actions, int position,
            int driveMode, boolean holdPosition) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiSetMotorPosition(position,
                    driveMode, holdPosition);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Motor Position")
    public @ActionOutput(name = "position", type = "int") @ActionOutput(name = "driveMode", type = "int") @ActionOutput(name = "holdPosition", type = "boolean") @ActionOutput(name = "positionReached", type = "boolean") Map<String, Object> brickletMotorizedLinearPotiGetMotorPosition()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMotorizedLinearPoti.MotorPosition value = dev.getMotorPosition();

        result.put("position", value.position);
        result.put("driveMode", value.driveMode);
        result.put("holdPosition", value.holdPosition);
        result.put("positionReached", value.positionReached);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetMotorPosition(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetMotorPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletMotorizedLinearPotiReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletMotorizedLinearPotiGetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletMotorizedLinearPotiGetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletMotorizedLinearPotiGetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMotorizedLinearPoti.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletMotorizedLinearPotiGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletMotorizedLinearPotiWrapper dev = ((@Nullable BrickletMotorizedLinearPotiWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletMotorizedLinearPoti.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }
}
