/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletNFCRFID;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the NFC/RFID Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletNFCRFIDActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Request Tag ID")
    public void brickletNFCRFIDRequestTagID(@ActionInput(name = "tagType") short tagType) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.requestTagID(tagType);
    }

    public static void brickletNFCRFIDRequestTagID(@Nullable ThingActions actions, short tagType)
            throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDRequestTagID(tagType);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get Tag ID")
    public @ActionOutput(name = "tagType", type = "short") @ActionOutput(name = "tidLength", type = "short") @ActionOutput(name = "tid", type = "short[]") Map<String, Object> brickletNFCRFIDGetTagID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletNFCRFID.TagID value = dev.getTagID();

        result.put("tagType", value.tagType);
        result.put("tidLength", value.tidLength);
        result.put("tid", value.tid);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetTagID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetTagID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get State")
    public @ActionOutput(name = "state", type = "short") @ActionOutput(name = "idle", type = "boolean") Map<String, Object> brickletNFCRFIDGetState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletNFCRFID.State value = dev.getState();

        result.put("state", value.state);
        result.put("idle", value.idle);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Authenticate Mifare Classic Page")
    public void brickletNFCRFIDAuthenticateMifareClassicPage(@ActionInput(name = "page") int page,
            @ActionInput(name = "keyNumber") short keyNumber, @ActionInput(name = "key") short[] key)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.authenticateMifareClassicPage(page, keyNumber, key);
    }

    public static void brickletNFCRFIDAuthenticateMifareClassicPage(@Nullable ThingActions actions, int page,
            short keyNumber, short[] key) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDAuthenticateMifareClassicPage(page, keyNumber, key);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Write Page")
    public void brickletNFCRFIDWritePage(@ActionInput(name = "page") int page, @ActionInput(name = "data") short[] data)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.writePage(page, data);
    }

    public static void brickletNFCRFIDWritePage(@Nullable ThingActions actions, int page, short[] data)
            throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDWritePage(page, data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Request Page")
    public void brickletNFCRFIDRequestPage(@ActionInput(name = "page") int page) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.requestPage(page);
    }

    public static void brickletNFCRFIDRequestPage(@Nullable ThingActions actions, int page)
            throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDRequestPage(page);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get Page")
    public @ActionOutput(name = "data", type = "short[]") Map<String, Object> brickletNFCRFIDGetPage()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        short[] value = dev.getPage();

        result.put("data", value);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetPage(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetPage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletNFCRFIDGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletNFCRFIDWrapper dev = ((@Nullable BrickletNFCRFIDWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletNFCRFID.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }
}
