/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletRGBLED;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the RGB LED Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRGBLEDActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set RGB Value")
    public void brickletRGBLEDSetRGBValue(@ActionInput(name = "r") short r, @ActionInput(name = "g") short g,
            @ActionInput(name = "b") short b) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletRGBLEDWrapper dev = ((@Nullable BrickletRGBLEDWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setRGBValue(r, g, b);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletRGBLEDColor")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletRGBLEDColor"), RefreshType.REFRESH);
        }
    }

    public static void brickletRGBLEDSetRGBValue(@Nullable ThingActions actions, short r, short g, short b)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDActions) {
            ((BrickletRGBLEDActions) actions).brickletRGBLEDSetRGBValue(r, g, b);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDActions class.");
        }
    }

    @RuleAction(label = "Get RGB Value")
    public @ActionOutput(name = "r", type = "short") @ActionOutput(name = "g", type = "short") @ActionOutput(name = "b", type = "short") Map<String, Object> brickletRGBLEDGetRGBValue()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDWrapper dev = ((@Nullable BrickletRGBLEDWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLED.RGBValue value = dev.getRGBValue();

        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDGetRGBValue(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDActions) {
            return ((BrickletRGBLEDActions) actions).brickletRGBLEDGetRGBValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletRGBLEDGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDWrapper dev = ((@Nullable BrickletRGBLEDWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLED.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDActions) {
            return ((BrickletRGBLEDActions) actions).brickletRGBLEDGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDActions class.");
        }
    }
}
