/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletRGBLEDButton;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the RGB LED Button Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRGBLEDButtonActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Get Color")
    public @ActionOutput(name = "red", type = "int") @ActionOutput(name = "green", type = "int") @ActionOutput(name = "blue", type = "int") Map<String, Object> brickletRGBLEDButtonGetColor()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLEDButton.Color value = dev.getColor();

        result.put("red", value.red);
        result.put("green", value.green);
        result.put("blue", value.blue);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetColor(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetColor();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "state", type = "int") Map<String, Object> brickletRGBLEDButtonGetButtonState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getButtonState();

        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetButtonState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetButtonState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Color Calibration")
    public @ActionOutput(name = "red", type = "int") @ActionOutput(name = "green", type = "int") @ActionOutput(name = "blue", type = "int") Map<String, Object> brickletRGBLEDButtonGetColorCalibration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLEDButton.ColorCalibration value = dev.getColorCalibration();

        result.put("red", value.red);
        result.put("green", value.green);
        result.put("blue", value.blue);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetColorCalibration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetColorCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Set Color")
    public void brickletRGBLEDButtonSetColor(@ActionInput(name = "red") int red, @ActionInput(name = "green") int green,
            @ActionInput(name = "blue") int blue) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setColor(red, green, blue);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletRGBLEDButtonColor")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletRGBLEDButtonColor"), RefreshType.REFRESH);
        }
    }

    public static void brickletRGBLEDButtonSetColor(@Nullable ThingActions actions, int red, int green, int blue)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonSetColor(red, green, blue);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletRGBLEDButtonReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletRGBLEDButtonGetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletRGBLEDButtonGetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletRGBLEDButtonGetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLEDButton.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletRGBLEDButtonGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRGBLEDButtonWrapper dev = ((@Nullable BrickletRGBLEDButtonWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRGBLEDButton.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDButtonGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).brickletRGBLEDButtonGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }
}
