/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.HSBType;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletRGBLEDButton;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the RGB LED Button Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletRGBLEDButtonWrapper extends BrickletRGBLEDButton implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletRGBLEDButton.DEVICE_DISPLAY_NAME,
            "brickletrgbledbutton", BrickletRGBLEDButton.DEVICE_IDENTIFIER, BrickletRGBLEDButtonWrapper.class,
            BrickletRGBLEDButtonActions.class, "2.0.0", true);

    public BrickletRGBLEDButtonWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletRGBLEDButtonWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletRGBLEDButtonConfig cfg = (BrickletRGBLEDButtonConfig) config.as(BrickletRGBLEDButtonConfig.class);
        this.addButtonStateChangedListener(this.reg((state) -> {
            if (true) {
                triggerChannelFn.accept("BrickletRGBLEDButtonButton",
                        transformBrickletRGBLEDButtonButtonCallback0(state, cfg));
            }
        }, this::removeButtonStateChangedListener));

        this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletRGBLEDButtonColor");
        result.add("BrickletRGBLEDButtonButton");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletRGBLEDButtonColor":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletRGBLEDButtonColor"), "LED Color", "Color")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRGBLEDButtonColor"))
                        .withDescription("The color of the LED.").build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge RGB LED Button Bricklet").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Push button with built-in RGB LED")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletRGBLEDButtonColor",
                                new ChannelTypeUID("tinkerforge", "BrickletRGBLEDButtonColor")).withLabel("LED Color")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletRGBLEDButtonButton",
                                new ChannelTypeUID("system", "rawbutton")).withLabel("Button")
                                        .withDescription("This channel triggers if the button is pressed or released")
                                        .build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletrgbledbutton":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletRGBLEDButtonColor":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList())
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletRGBLEDButtonConfig cfg = (BrickletRGBLEDButtonConfig) config.as(BrickletRGBLEDButtonConfig.class);
        switch (channel) {
            case "BrickletRGBLEDButtonColor": {
                BrickletRGBLEDButton.Color value = this.getColor();
                if (true)
                    updateStateFn.accept(channel, transformBrickletRGBLEDButtonColorGetter0(value, cfg));
            }
                return;
            case "BrickletRGBLEDButtonButton": {
                int value = this.getButtonState();
                if (true)
                    triggerChannelFn.accept(channel, transformBrickletRGBLEDButtonButtonGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            case "BrickletRGBLEDButtonColor":
                if (command instanceof HSBType) {
                    HSBType cmd = (HSBType) command;
                    this.setColor((int) (cmd.getRed().doubleValue() * 255.0 / 100.0),
                            (int) (cmd.getGreen().doubleValue() * 255.0 / 100.0),
                            (int) (cmd.getBlue().doubleValue() * 255.0 / 100.0));
                }

                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of HSBType.",
                            command.getClass().getName(), channel);
                }

                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private String transformBrickletRGBLEDButtonButtonCallback0(int state, BrickletRGBLEDButtonConfig cfg) {
        return state == BrickletRGBLEDButton.BUTTON_STATE_PRESSED ? CommonTriggerEvents.PRESSED
                : CommonTriggerEvents.RELEASED;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletRGBLEDButtonColorGetter0(
            BrickletRGBLEDButton.Color value, BrickletRGBLEDButtonConfig cfg) {
        return HSBType.fromRGB(value.red, value.green, value.blue);
    }

    private String transformBrickletRGBLEDButtonButtonGetter0(int value, BrickletRGBLEDButtonConfig cfg) {
        return value == BrickletRGBLEDButton.BUTTON_STATE_PRESSED ? CommonTriggerEvents.PRESSED
                : CommonTriggerEvents.RELEASED;
    }
}
