/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletRS232V2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the RS232 Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRS232V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Write")
    public @ActionOutput(name = "messageWritten", type = "int") Map<String, Object> brickletRS232V2Write(
            @ActionInput(name = "message") char[] message) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.write(message);

        result.put("messageWritten", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2Write(@Nullable ThingActions actions, char[] message)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2Write(message);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Read")
    public @ActionOutput(name = "message", type = "char[]") Map<String, Object> brickletRS232V2Read(
            @ActionInput(name = "length") int length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        char[] value = dev.read(length);

        result.put("message", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2Read(@Nullable ThingActions actions, int length)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2Read(length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "baudrate", type = "long") @ActionOutput(name = "parity", type = "int") @ActionOutput(name = "stopbits", type = "int") @ActionOutput(name = "wordlength", type = "int") @ActionOutput(name = "flowcontrol", type = "int") Map<String, Object> brickletRS232V2GetConfiguration()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRS232V2.Configuration value = dev.getConfiguration();

        result.put("baudrate", value.baudrate);
        result.put("parity", value.parity);
        result.put("stopbits", value.stopbits);
        result.put("wordlength", value.wordlength);
        result.put("flowcontrol", value.flowcontrol);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetConfiguration(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Buffer Config")
    public @ActionOutput(name = "sendBufferSize", type = "int") @ActionOutput(name = "receiveBufferSize", type = "int") Map<String, Object> brickletRS232V2GetBufferConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRS232V2.BufferConfig value = dev.getBufferConfig();

        result.put("sendBufferSize", value.sendBufferSize);
        result.put("receiveBufferSize", value.receiveBufferSize);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetBufferConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetBufferConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Buffer Status")
    public @ActionOutput(name = "sendBufferUsed", type = "int") @ActionOutput(name = "receiveBufferUsed", type = "int") Map<String, Object> brickletRS232V2GetBufferStatus()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRS232V2.BufferStatus value = dev.getBufferStatus();

        result.put("sendBufferUsed", value.sendBufferUsed);
        result.put("receiveBufferUsed", value.receiveBufferUsed);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetBufferStatus(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetBufferStatus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Error Count")
    public @ActionOutput(name = "errorCountOverrun", type = "long") @ActionOutput(name = "errorCountParity", type = "long") Map<String, Object> brickletRS232V2GetErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRS232V2.ErrorCount value = dev.getErrorCount();

        result.put("errorCountOverrun", value.errorCountOverrun);
        result.put("errorCountParity", value.errorCountParity);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletRS232V2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletRS232V2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletRS232V2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletRS232V2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRS232V2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletRS232V2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletRS232V2Wrapper dev = ((@Nullable BrickletRS232V2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletRS232V2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }
}
