/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletSolidStateRelay;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Solid State Relay Bricklet.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSolidStateRelayActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set State")
    public void brickletSolidStateRelaySetState(@ActionInput(name = "state") boolean state)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSolidStateRelayWrapper dev = ((@Nullable BrickletSolidStateRelayWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setState(state);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayRelay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayRelay"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayMonoflopRelay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayMonoflopRelay"), RefreshType.REFRESH);
        }
    }

    public static void brickletSolidStateRelaySetState(@Nullable ThingActions actions, boolean state)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayActions) {
            ((BrickletSolidStateRelayActions) actions).brickletSolidStateRelaySetState(state);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayActions class.");
        }
    }

    @RuleAction(label = "Get State")
    public @ActionOutput(name = "state", type = "boolean") Map<String, Object> brickletSolidStateRelayGetState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayWrapper dev = ((@Nullable BrickletSolidStateRelayWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.getState();

        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayGetState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayActions) {
            return ((BrickletSolidStateRelayActions) actions).brickletSolidStateRelayGetState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayActions class.");
        }
    }

    @RuleAction(label = "Set Monoflop")
    public void brickletSolidStateRelaySetMonoflop(@ActionInput(name = "state") boolean state,
            @ActionInput(name = "time") long time) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSolidStateRelayWrapper dev = ((@Nullable BrickletSolidStateRelayWrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setMonoflop(state, time);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayRelay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayRelay"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayMonoflopRelay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayMonoflopRelay"), RefreshType.REFRESH);
        }
    }

    public static void brickletSolidStateRelaySetMonoflop(@Nullable ThingActions actions, boolean state, long time)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayActions) {
            ((BrickletSolidStateRelayActions) actions).brickletSolidStateRelaySetMonoflop(state, time);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayActions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "state", type = "boolean") @ActionOutput(name = "time", type = "long") @ActionOutput(name = "timeRemaining", type = "long") Map<String, Object> brickletSolidStateRelayGetMonoflop()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayWrapper dev = ((@Nullable BrickletSolidStateRelayWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSolidStateRelay.Monoflop value = dev.getMonoflop();

        result.put("state", value.state);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayGetMonoflop(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayActions) {
            return ((BrickletSolidStateRelayActions) actions).brickletSolidStateRelayGetMonoflop();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletSolidStateRelayGetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayWrapper dev = ((@Nullable BrickletSolidStateRelayWrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSolidStateRelay.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayGetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayActions) {
            return ((BrickletSolidStateRelayActions) actions).brickletSolidStateRelayGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayActions class.");
        }
    }
}
