/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import com.tinkerforge.BrickletSolidStateRelayV2;
import com.tinkerforge.TinkerforgeException;

/**
 * Passes through actions of the Solid State Relay Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSolidStateRelayV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) {
        this.handler = (DeviceHandler) handler;
    }

    @Override
    public @Nullable ThingHandler getThingHandler() {
        return handler;
    }

    @RuleAction(label = "Set State")
    public void brickletSolidStateRelayV2SetState(@ActionInput(name = "state") boolean state)
            throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setState(state);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2Relay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2Relay"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2MonoflopRelay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2MonoflopRelay"), RefreshType.REFRESH);
        }
    }

    public static void brickletSolidStateRelayV2SetState(@Nullable ThingActions actions, boolean state)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2SetState(state);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get State")
    public @ActionOutput(name = "state", type = "boolean") Map<String, Object> brickletSolidStateRelayV2GetState()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        boolean value = dev.getState();

        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetState(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Set Monoflop")
    public void brickletSolidStateRelayV2SetMonoflop(@ActionInput(name = "state") boolean state,
            @ActionInput(name = "time") long time) throws TinkerforgeException {
        DeviceHandler h = handler;
        if (h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return;
        }
        dev.setMonoflop(state, time);
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2Relay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2Relay"), RefreshType.REFRESH);
        }
        if (t.getChannel(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2MonoflopRelay")) != null) {
            h.handleCommand(new ChannelUID(t.getUID(), "BrickletSolidStateRelayV2MonoflopRelay"), RefreshType.REFRESH);
        }
    }

    public static void brickletSolidStateRelayV2SetMonoflop(@Nullable ThingActions actions, boolean state, long time)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2SetMonoflop(state, time);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "state", type = "boolean") @ActionOutput(name = "time", type = "long") @ActionOutput(name = "timeRemaining", type = "long") Map<String, Object> brickletSolidStateRelayV2GetMonoflop()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSolidStateRelayV2.Monoflop value = dev.getMonoflop();

        result.put("state", value.state);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetMonoflop(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetMonoflop();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type = "long") Map<String, Object> brickletSolidStateRelayV2ReadUID()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        long value = dev.readUID();

        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2ReadUID(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type = "int") Map<String, Object> brickletSolidStateRelayV2GetChipTemperature()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();

        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetChipTemperature(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type = "int") Map<String, Object> brickletSolidStateRelayV2GetStatusLEDConfig()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();

        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetStatusLEDConfig(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type = "long") @ActionOutput(name = "errorCountMessageChecksum", type = "long") @ActionOutput(name = "errorCountFrame", type = "long") @ActionOutput(name = "errorCountOverflow", type = "long") Map<String, Object> brickletSolidStateRelayV2GetSPITFPErrorCount()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSolidStateRelayV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();

        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetSPITFPErrorCount(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type = "String") @ActionOutput(name = "connectedUid", type = "String") @ActionOutput(name = "position", type = "char") @ActionOutput(name = "hardwareVersion", type = "short[]") @ActionOutput(name = "firmwareVersion", type = "short[]") @ActionOutput(name = "deviceIdentifier", type = "int") Map<String, Object> brickletSolidStateRelayV2GetIdentity()
            throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if (h == null) {
            return result;
        }

        @Nullable
        BrickletSolidStateRelayV2Wrapper dev = ((@Nullable BrickletSolidStateRelayV2Wrapper) h.getDevice());
        if (dev == null) {
            return result;
        }

        BrickletSolidStateRelayV2.Identity value = dev.getIdentity();

        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetIdentity(@Nullable ThingActions actions)
            throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }
}
