/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletUVLightV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the UV Light Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletUVLightV2Wrapper extends BrickletUVLightV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletUVLightV2.DEVICE_DISPLAY_NAME,
            "brickletuvlightv2", BrickletUVLightV2.DEVICE_IDENTIFIER, BrickletUVLightV2Wrapper.class,
            BrickletUVLightV2Actions.class, "2.0.0", true);

    public BrickletUVLightV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletUVLightV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletUVLightV2Config cfg = (BrickletUVLightV2Config) config.as(BrickletUVLightV2Config.class);
        this.addUVAListener(this.reg((uva) -> {
            if (true) {
                updateStateFn.accept("BrickletUVLightV2UVA", transformBrickletUVLightV2UVACallback0(uva, cfg));
            }
        }, this::removeUVAListener));
        this.addUVBListener(this.reg((uvb) -> {
            if (true) {
                updateStateFn.accept("BrickletUVLightV2UVB", transformBrickletUVLightV2UVBCallback0(uvb, cfg));
            }
        }, this::removeUVBListener));
        this.addUVIListener(this.reg((uvi) -> {
            if (true) {
                updateStateFn.accept("BrickletUVLightV2UVI", transformBrickletUVLightV2UVICallback0(uvi, cfg));
            }
        }, this::removeUVIListener));
        this.setConfiguration(cfg.integrationTime);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletUVLightV2UVAConfig channelCfg = getChannelConfigFn.apply("BrickletUVLightV2UVA")
                    .as(BrickletUVLightV2UVAConfig.class);
            this.setUVACallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletUVLightV2UVBConfig channelCfg = getChannelConfigFn.apply("BrickletUVLightV2UVB")
                    .as(BrickletUVLightV2UVBConfig.class);
            this.setUVBCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletUVLightV2UVIConfig channelCfg = getChannelConfigFn.apply("BrickletUVLightV2UVI")
                    .as(BrickletUVLightV2UVIConfig.class);
            this.setUVICallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setUVACallbackConfiguration(0, true, 'x', 0, 0);
        this.setUVBCallbackConfiguration(0, true, 'x', 0, 0);
        this.setUVICallbackConfiguration(0, true, 'x', 0, 0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletUVLightV2UVA");
        result.add("BrickletUVLightV2UVB");
        result.add("BrickletUVLightV2UVI");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletUVLightV2UVA":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletUVLightV2UVA"), "UV-A Intensity",
                                "Number:Intensity")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletUVLightV2UVA"))
                        .withDescription(
                                "The sensor has not weighted the intensity with the erythemal action spectrum to get the skin-affecting irradiation. Therefore, you cannot just divide the value by 250 to get the UVA index.")
                        .withStateDescription(StateDescriptionFragmentBuilder.create()
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(214748.3647))
                                .withStep(BigDecimal.valueOf(9.999999995343387e-05)).withPattern("%.5f %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletUVLightV2UVB":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletUVLightV2UVB"), "UV-B Intensity",
                                "Number:Intensity")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletUVLightV2UVB"))
                        .withDescription(
                                "The sensor has not weighted the intensity with the erythemal action spectrum to get the skin-affecting irradiation. Therefore, you cannot just divide the value by 250 to get the UVB index.")
                        .withStateDescription(StateDescriptionFragmentBuilder.create()
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(214748.3647))
                                .withStep(BigDecimal.valueOf(9.999999995343387e-05)).withPattern("%.5f %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletUVLightV2UVI":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletUVLightV2UVI"), "UV Index",
                                "Number:Dimensionless")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletUVLightV2UVI"))
                        .withDescription("Calculated UV Index")
                        .withStateDescription(StateDescriptionFragmentBuilder.create()
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(50))
                                .withStep(BigDecimal.valueOf(2.3283064365386963e-08)).withPattern("%.3f %unit%")
                                .withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge UV Light Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures UV-A, UV-B and UV index")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletUVLightV2UVA",
                                new ChannelTypeUID("tinkerforge", "BrickletUVLightV2UVA")).withLabel("UV-A Intensity")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletUVLightV2UVB",
                                new ChannelTypeUID("tinkerforge", "BrickletUVLightV2UVB")).withLabel("UV-B Intensity")
                                        .build(),
                        new ChannelDefinitionBuilder("BrickletUVLightV2UVI",
                                new ChannelTypeUID("tinkerforge", "BrickletUVLightV2UVI")).withLabel("UV Index")
                                        .build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletuvlightv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("integrationTime", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The integration time can be configured between 50 and 800 ms. With a shorter integration time the sensor reading updates more often but contains more noise. With a longer integration the sensor reading contains less noise but updates less often.<br/><br/>With a longer integration time (especially 800 ms) and a higher UV intensity the sensor can be saturated. If this happens the UVA/UVB/UVI readings are all -1. In this case you need to choose a shorter integration time.")
                                .withLabel("Integration Time").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "50ms"),
                                        new ParameterOption("1", "100ms"), new ParameterOption("2", "200ms"),
                                        new ParameterOption("3", "400ms"), new ParameterOption("4", "800ms")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletUVLightV2UVA":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletUVLightV2UVB":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletUVLightV2UVI":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletUVLightV2Config cfg = (BrickletUVLightV2Config) config.as(BrickletUVLightV2Config.class);
        switch (channel) {
            case "BrickletUVLightV2UVA": {
                int value = this.getUVA();
                if (true)
                    updateStateFn.accept(channel, transformBrickletUVLightV2UVAGetter0(value, cfg));
            }
                return;
            case "BrickletUVLightV2UVB": {
                int value = this.getUVB();
                if (true)
                    updateStateFn.accept(channel, transformBrickletUVLightV2UVBGetter0(value, cfg));
            }
                return;
            case "BrickletUVLightV2UVI": {
                int value = this.getUVI();
                if (true)
                    updateStateFn.accept(channel, transformBrickletUVLightV2UVIGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletUVLightV2UVACallback0(int uva,
            BrickletUVLightV2Config cfg) {
        return new QuantityType<>(uva / 10000.0, SmartHomeUnits.IRRADIANCE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletUVLightV2UVBCallback0(int uvb,
            BrickletUVLightV2Config cfg) {
        return new QuantityType<>(uvb / 10000.0, SmartHomeUnits.IRRADIANCE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletUVLightV2UVICallback0(int uvi,
            BrickletUVLightV2Config cfg) {
        return new DecimalType(uvi / 10.0);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletUVLightV2UVAGetter0(int value,
            BrickletUVLightV2Config cfg) {
        return new QuantityType<>(value / 10000.0, SmartHomeUnits.IRRADIANCE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletUVLightV2UVBGetter0(int value,
            BrickletUVLightV2Config cfg) {
        return new QuantityType<>(value / 10000.0, SmartHomeUnits.IRRADIANCE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletUVLightV2UVIGetter0(int value,
            BrickletUVLightV2Config cfg) {
        return new DecimalType(value / 10.0);
    }
}
