/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

/* ***********************************************************
* This file was automatically generated on 2020-04-29.      *
*                                                           *
* openHAB Bindings Version 2.0.0                            *
*                                                           *
* If you have a bugfix for this file and want to commit it, *
* please fix the bug in the generator. You can find a link  *
* to the generators git repository on tinkerforge.com       *
*************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tinkerforge.BrickletVoltageCurrentV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

/**
 * Wraps the Java Bindings of the Voltage/Current Bricklet 2.0.
 * 
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletVoltageCurrentV2Wrapper extends BrickletVoltageCurrentV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletVoltageCurrentV2.DEVICE_DISPLAY_NAME,
            "brickletvoltagecurrentv2", BrickletVoltageCurrentV2.DEVICE_IDENTIFIER,
            BrickletVoltageCurrentV2Wrapper.class, BrickletVoltageCurrentV2Actions.class, "2.0.0", true);

    public BrickletVoltageCurrentV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    private final Logger logger = LoggerFactory.getLogger(BrickletVoltageCurrentV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config,
            Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler)
            throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletVoltageCurrentV2Config cfg = (BrickletVoltageCurrentV2Config) config
                .as(BrickletVoltageCurrentV2Config.class);
        this.addCurrentListener(this.reg((current) -> {
            if (true) {
                updateStateFn.accept("BrickletVoltageCurrentV2Current",
                        transformBrickletVoltageCurrentV2CurrentCallback0(current, cfg));
            }
        }, this::removeCurrentListener));
        this.addVoltageListener(this.reg((voltage) -> {
            if (true) {
                updateStateFn.accept("BrickletVoltageCurrentV2Voltage",
                        transformBrickletVoltageCurrentV2VoltageCallback0(voltage, cfg));
            }
        }, this::removeVoltageListener));
        this.addPowerListener(this.reg((power) -> {
            if (true) {
                updateStateFn.accept("BrickletVoltageCurrentV2Power",
                        transformBrickletVoltageCurrentV2PowerCallback0(power, cfg));
            }
        }, this::removePowerListener));
        this.setConfiguration(cfg.averaging, cfg.voltageConversionTime, cfg.currentConversionTime);
        this.setStatusLEDConfig(cfg.statusLEDConfig);
        {
            BrickletVoltageCurrentV2CurrentConfig channelCfg = getChannelConfigFn
                    .apply("BrickletVoltageCurrentV2Current").as(BrickletVoltageCurrentV2CurrentConfig.class);
            this.setCurrentCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletVoltageCurrentV2VoltageConfig channelCfg = getChannelConfigFn
                    .apply("BrickletVoltageCurrentV2Voltage").as(BrickletVoltageCurrentV2VoltageConfig.class);
            this.setVoltageCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
        {
            BrickletVoltageCurrentV2PowerConfig channelCfg = getChannelConfigFn.apply("BrickletVoltageCurrentV2Power")
                    .as(BrickletVoltageCurrentV2PowerConfig.class);
            this.setPowerCallbackConfiguration(channelCfg.updateInterval, true, 'x', 0, 0);
        }
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        this.setCurrentCallbackConfiguration(0, true, 'x', 0, 0);
        this.setVoltageCallbackConfiguration(0, true, 'x', 0, 0);
        this.setPowerCallbackConfiguration(0, true, 'x', 0, 0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config)
            throws TinkerforgeException {
        List<String> result = new ArrayList<String>();
        result.add("BrickletVoltageCurrentV2Current");
        result.add("BrickletVoltageCurrentV2Voltage");
        result.add("BrickletVoltageCurrentV2Power");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch (channelTypeUID.getId()) {
            case "BrickletVoltageCurrentV2Voltage":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletVoltageCurrentV2Voltage"), "Measured Voltage",
                                "Number:ElectricPotential")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletVoltageCurrentV2Voltage"))
                        .withDescription("The measured voltage between 0 and 36V.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(36)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletVoltageCurrentV2Current":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletVoltageCurrentV2Current"), "Measured Current",
                                "Number:ElectricCurrent")
                        .withConfigDescriptionURI(
                                URI.create("channel-type:tinkerforge:BrickletVoltageCurrentV2Current"))
                        .withDescription("The measured current between -20 and 20A.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-20))
                                        .withMaximum(BigDecimal.valueOf(20)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            case "BrickletVoltageCurrentV2Power":
                return ChannelTypeBuilder
                        .state(new ChannelTypeUID("tinkerforge", "BrickletVoltageCurrentV2Power"), "Measured Power",
                                "Number:Power")
                        .withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletVoltageCurrentV2Power"))
                        .withDescription("The measured power between 0 and 720W.")
                        .withStateDescription(
                                StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0))
                                        .withMaximum(BigDecimal.valueOf(720)).withStep(BigDecimal.valueOf(0.001))
                                        .withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription())
                        .build();
            default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build()
                .getProperties();
        Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
        props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Voltage/Current Bricklet 2.0").isListed(false)
                .withSupportedBridgeTypeUIDs(
                        Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString()))
                .withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId()))
                .withDescription("Measures power, DC voltage and DC current up to 720W/36V/20A")
                .withChannelDefinitions(Arrays.asList(
                        new ChannelDefinitionBuilder("BrickletVoltageCurrentV2Current",
                                new ChannelTypeUID("tinkerforge", "BrickletVoltageCurrentV2Current"))
                                        .withLabel("Measured Current").build(),
                        new ChannelDefinitionBuilder("BrickletVoltageCurrentV2Voltage",
                                new ChannelTypeUID("tinkerforge", "BrickletVoltageCurrentV2Voltage"))
                                        .withLabel("Measured Voltage").build(),
                        new ChannelDefinitionBuilder("BrickletVoltageCurrentV2Power",
                                new ChannelTypeUID("tinkerforge", "BrickletVoltageCurrentV2Power"))
                                        .withLabel("Measured Power").build()))
                .withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch (uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletvoltagecurrentv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(
                        ConfigDescriptionParameterBuilder.create("averaging", Type.INTEGER).withDefault("3")
                                .withDescription("Configures the number of samples to average over.")
                                .withLabel("Averaging").withLimitToOptions(true)
                                .withOptions(
                                        Arrays.asList(new ParameterOption("0", "1"), new ParameterOption("1", "4"),
                                                new ParameterOption("2", "16"), new ParameterOption("3", "64"),
                                                new ParameterOption("4", "128"), new ParameterOption("5", "256"),
                                                new ParameterOption("6", "512"), new ParameterOption("7", "1024")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("voltageConversionTime", Type.INTEGER).withDefault("4")
                                .withDescription("Configures the voltage conversion time.")
                                .withLabel("Voltage Conversion Time").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "140µs"),
                                        new ParameterOption("1", "204µs"), new ParameterOption("2", "332µs"),
                                        new ParameterOption("3", "588µs"), new ParameterOption("4", "1.1ms"),
                                        new ParameterOption("5", "2.116ms"), new ParameterOption("6", "4.156ms"),
                                        new ParameterOption("7", "8.244ms")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("currentConversionTime", Type.INTEGER).withDefault("4")
                                .withDescription("Configures the current conversion time.")
                                .withLabel("Current Conversion Time").withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "140µs"),
                                        new ParameterOption("1", "204µs"), new ParameterOption("2", "332µs"),
                                        new ParameterOption("3", "588µs"), new ParameterOption("4", "1.1ms"),
                                        new ParameterOption("5", "2.116ms"), new ParameterOption("6", "4.156ms"),
                                        new ParameterOption("7", "8.244ms")))
                                .build(),
                        ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3")
                                .withDescription(
                                        "The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.")
                                .withLabel("Status LED Config").withAdvanced(true).withLimitToOptions(true)
                                .withOptions(Arrays.asList(new ParameterOption("0", "Off"),
                                        new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"),
                                        new ParameterOption("3", "Show Status")))
                                .build()))
                        .withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals",
                                "null", true, "Update Intervals", "Update Intervals")))
                        .build();
            case "channel-type:tinkerforge:BrickletVoltageCurrentV2Voltage":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletVoltageCurrentV2Current":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletVoltageCurrentV2Power":
                return ConfigDescriptionBuilder.create(uri)
                        .withParameters(Arrays.asList(ConfigDescriptionParameterBuilder
                                .create("updateInterval", Type.INTEGER).withDefault("1000")
                                .withDescription(
                                        "Specifies the update interval in milliseconds. A value of 0 disables automatic updates.")
                                .withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms")
                                .withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L))
                                .build()))
                        .withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

    @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig,
            BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn,
            BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletVoltageCurrentV2Config cfg = (BrickletVoltageCurrentV2Config) config
                .as(BrickletVoltageCurrentV2Config.class);
        switch (channel) {
            case "BrickletVoltageCurrentV2Current": {
                int value = this.getCurrent();
                if (true)
                    updateStateFn.accept(channel, transformBrickletVoltageCurrentV2CurrentGetter0(value, cfg));
            }
                return;
            case "BrickletVoltageCurrentV2Voltage": {
                int value = this.getVoltage();
                if (true)
                    updateStateFn.accept(channel, transformBrickletVoltageCurrentV2VoltageGetter0(value, cfg));
            }
                return;
            case "BrickletVoltageCurrentV2Power": {
                int value = this.getPower();
                if (true)
                    updateStateFn.accept(channel, transformBrickletVoltageCurrentV2PowerGetter0(value, cfg));
            }
                return;
            default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }

    @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config,
            org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command)
            throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();

        switch (channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

    private org.eclipse.smarthome.core.types.State transformBrickletVoltageCurrentV2CurrentCallback0(int current,
            BrickletVoltageCurrentV2Config cfg) {
        return new QuantityType<>(current / 1000.0, SmartHomeUnits.AMPERE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletVoltageCurrentV2VoltageCallback0(int voltage,
            BrickletVoltageCurrentV2Config cfg) {
        return new QuantityType<>(voltage / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletVoltageCurrentV2PowerCallback0(int power,
            BrickletVoltageCurrentV2Config cfg) {
        return new QuantityType<>(power / 1000.0, SmartHomeUnits.WATT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletVoltageCurrentV2CurrentGetter0(int value,
            BrickletVoltageCurrentV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.AMPERE);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletVoltageCurrentV2VoltageGetter0(int value,
            BrickletVoltageCurrentV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }

    private org.eclipse.smarthome.core.types.State transformBrickletVoltageCurrentV2PowerGetter0(int value,
            BrickletVoltageCurrentV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.WATT);
    }
}
